/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package algo.structures

import scala.collection.mutable.HashSet
import scala.collection.mutable.HashMap
import algo.Generator1

class CommunitiesInformation(gen : Generator1) {
	val communitiesInformation = (for(k <- 0 until gen.k) yield {
	    val communityVertices = HashSet[Int]((0 until gen.nbVertices).toList.filter(gen.graph.vertexCommunities(_) == k) : _*)
	    val nbInternalEdges = (for(v <- communityVertices.toList) yield (gen.graph.graphTab(v).intersect(communityVertices).size)).sum / 2
	    val nbExternalEdges = (for(v <- communityVertices.toList) yield (gen.graph.graphTab(v).diff(communityVertices).size)).sum
	    
	    // Compute min, max and average value of the attributes for each communities
	    val nbAttr = gen.dev.length
	    val minAttrs = Array.fill[Double](nbAttr)(Double.MaxValue)
	    val maxAttrs = Array.fill[Double](nbAttr)(Double.MinValue)
	    val avgAttrs = Array.fill[Double](nbAttr)(0D)
	    for(v <- communityVertices; i <- 0 until nbAttr) {
	        minAttrs(i) = math.min(gen.graph.attr(v)(i), minAttrs(i))
	        maxAttrs(i) = math.max(gen.graph.attr(v)(i), maxAttrs(i))
	        avgAttrs(i) += gen.graph.attr(v)(i)
	    }
	    for(i <- 0 until nbAttr) { avgAttrs(i) /= communityVertices.size }
	    
	    // Compute the degree distribution for each community
	    val degDistMap = HashMap[Int, Int]()
        for(v <- communityVertices) { degDistMap(gen.graph.degree(v)) = degDistMap.getOrElse(gen.graph.degree(v), 0) + 1 }
	    val degreeDistribution = degDistMap.toArray.sortBy(a => a._1)
	    
	    CommunityInformation(gen.k, k, communityVertices, degreeDistribution, nbInternalEdges, nbExternalEdges, minAttrs, maxAttrs, avgAttrs)
	}).toArray
}

case class CommunityInformation(k : Int, idx : Int, vertices : HashSet[Int], degreeDistribution : Array[(Int, Int)], 
        nbInternalEdges : Int, nbExternalEdges : Int, 
        minAttrs : Array[Double], maxAttrs : Array[Double], avgAttrs : Array[Double])