/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package gui

import java.awt.BorderLayout
import java.awt.Color
import java.awt.Dimension
import java.awt.FlowLayout
import java.awt.Graphics
import java.awt.Graphics2D
import java.awt.GridLayout
import java.awt.RenderingHints
import algo.structures.CommunitiesInformation
import algo.structures.CommunityInformation
import algo.Generator1
import javax.swing.BoxLayout
import javax.swing.JLabel
import javax.swing.JPanel
import javax.swing.JScrollPane
import utils.Colors
import java.awt.ScrollPane
import javax.swing.ScrollPaneConstants
import gui.components.DegreeDistribution

class CommunityInformationPanel(communityInformation : CommunityInformation) extends JPanel {
    setLayout(new BorderLayout())
    
	// Build the degree distribution panel
    // val degreeDistributionPanel = new DegreeDistrib(communityInformation.degreeDistribution, Option(new Dimension(200, 200)))
    val degreeDistributionPanel = new DegreeDistribution(None, communityInformation.degreeDistribution)

	// Build the attribute information panel
	val attrPanel = new JPanel(new GridLayout(0, 4))
	attrPanel.add(new JLabel(""))
	attrPanel.add(new JLabel("<html><b>Min"))
	attrPanel.add(new JLabel("<html><b>Max"))
	attrPanel.add(new JLabel("<html><b>Avg"))
	for(i <- 0 until communityInformation.avgAttrs.length) {
	    attrPanel.add(new JLabel(s"<html><b>Attr ${i + 1}      "))
		attrPanel.add(new JLabel(f"${communityInformation.minAttrs(i)}%.3f"))
		attrPanel.add(new JLabel(f"${communityInformation.maxAttrs(i)}%.3f"))
		attrPanel.add(new JLabel(f"${communityInformation.avgAttrs(i)}%.3f"))
	}
    attrPanel.setBackground(Color.white)
    
    val structuralInformationPanel = new JPanel()
    structuralInformationPanel.setLayout(new BoxLayout(structuralInformationPanel, BoxLayout.PAGE_AXIS))
    structuralInformationPanel.add(new JLabel("Nb vertices: " + communityInformation.vertices.size))
    structuralInformationPanel.add(new JLabel("External edges: " + communityInformation.nbExternalEdges))
    structuralInformationPanel.add(new JLabel("Internal edges: " + communityInformation.nbInternalEdges))
    structuralInformationPanel.add(new JLabel(" "))
    structuralInformationPanel.add(new JLabel(" "))
    structuralInformationPanel.add(attrPanel)
    structuralInformationPanel.setBackground(Color.white)
    
    val titlePanel = new JPanel(new FlowLayout(FlowLayout.LEFT))
    titlePanel.add(new SmallVertexPanel(Colors.getMatchingColors(communityInformation.k, value = 0.6f)(communityInformation.idx)))
    titlePanel.add(new JLabel(s"<html><b>Community ${communityInformation.idx + 1}</b></html>"))
    titlePanel.setBackground(Colors.getMatchingColors(communityInformation.k)(communityInformation.idx))
    
    val infoPanel = new JPanel(new FlowLayout(FlowLayout.LEFT))
    infoPanel.add(structuralInformationPanel, BorderLayout.EAST)
    infoPanel.add(degreeDistributionPanel, BorderLayout.CENTER)
    infoPanel.setBackground(Color.white)
    setBackground(Color.white)
    
    add(titlePanel, BorderLayout.NORTH)
    add(infoPanel, BorderLayout.CENTER)
    
}

class SmallVertexPanel(color : Color) extends JPanel {
    override def paint(g : Graphics) {
        val g2d = g.asInstanceOf[Graphics2D]
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g2d.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
        g2d.setColor(color)
        g2d.fillOval(0, 0, 9, 9)
        g2d.setColor(Color.black)
        g2d.drawOval(0, 0, 9, 9)
    }
}

class CommunitiesInformationPanel() extends JPanel(new BorderLayout) {
    val container = new JPanel()
    container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS))
    val scrollPane = new JScrollPane(container, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED)
    add(scrollPane, BorderLayout.CENTER)
    def setGenerator(gen : Generator1) {
        container.removeAll()
        val comunitiesInformation = new CommunitiesInformation(gen).communitiesInformation
        for(communityInformation <- comunitiesInformation) { container.add(new CommunityInformationPanel(communityInformation)) }
        repaint()
    }
}