/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package gui

import java.awt.BorderLayout
import java.awt.FlowLayout
import java.awt.event.ActionListener
import algo.Generator1
import javax.swing.JButton
import javax.swing.JFrame
import javax.swing.JLabel
import javax.swing.JPanel
import javax.swing.JSpinner
import javax.swing.JTextField
import javax.swing.SpinnerNumberModel
import java.awt.event.ActionEvent
import javax.swing.UIManager
import javax.swing.JDialog
import javax.swing.Box
import java.util.prefs.Preferences
import java.awt.event.WindowListener
import java.awt.event.WindowEvent
import java.awt.event.WindowAdapter
import scala.collection.mutable.HashSet
import javax.swing.JSplitPane
import java.awt.Color
import gui.components.SpinSlider
import com.alee.laf.menu.WebMenuBar
import com.alee.laf.label.WebLabel
import java.awt.Insets
import com.alee.laf.menu.MenuBarStyle
import com.alee.laf.button.WebButton
import com.alee.laf.panel.WebPanel
import gui.graph.GraphLayoutPanel
import gui.graph.GraphView
import gui.components.Savable
import gui.components.SavableManager
import gui.components.DegreeDistribution
import utils.Icon

object Main extends JFrame with Savable {
    var measuresView : MeasurePanel = _
    var degreeDistribution = new DegreeDistribution()
    val communitiesInformationPanel = new CommunitiesInformationPanel()
    
    setIconImage(Icon.get("favicon").getImage())
    
    def main(args: Array[String]): Unit = {
        com.alee.laf.WebLookAndFeel.install()
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            def run() {
                JFrame.setDefaultLookAndFeelDecorated(true)
                JDialog.setDefaultLookAndFeelDecorated(true)
                createAndShowGUI()
            }
        })
    }
    
    def save() = {
        SavableManager.save("frameWidth", this.getWidth())
        SavableManager.save("frameHeight", this.getHeight())
        SavableManager.save("frameX", this.getX())
        SavableManager.save("frameY", this.getY())
    }
    
    def load() = {
        setLocation(SavableManager.load("frameX", 0), SavableManager.load("frameY", 0))
        setSize(SavableManager.load("frameWidth", 800), SavableManager.load("frameHeight", 600))
    }
    
    def createAndShowGUI() {
        setLayout(new BorderLayout())
        setTitle("Graph Generator")
        
        measuresView = new MeasurePanel
        addWindowListener(new WindowAdapter { override def windowClosing(e : WindowEvent) = { exit } })
        setVisible(true)
        
        var rootPanel = getContentPane()
	    rootPanel.setLayout(new BorderLayout())
	    rootPanel.add(ViewManager.station, BorderLayout.CENTER)
	    rootPanel.add(gui.panels.StatusBar, BorderLayout.SOUTH)
        
	    ViewManager.registerPanel(RunPanel.topPanel, "Parameters", Option("wrench"))
//	    ViewManager.registerPanel(GraphView, "Graph", Option("chart_organisation"))
	    ViewManager.registerPanel(measuresView, "Measures", Option("control_equalizer"))
//	    ViewManager.registerPanel(degreeDistribution, "Degree distribution", Option("chart_curve"))
	    ViewManager.registerPanel(GraphLayoutPanel, "GraphLayout", Option("chart_organisation"))
//	    ViewManager.registerPanel(communitiesInformationPanel, "Communities Information", Option("chart_line"))
	    ViewManager.loadLayout()
	    SavableManager.loadAll
    }
    
    def exit() = {
        SavableManager.saveAll
        ViewManager.saveLayout()
        System.exit(0)
    }
}

