/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package gui

import java.awt.BorderLayout
import java.awt.Color
import java.awt.FlowLayout
import java.awt.GridLayout
import java.awt.event.ActionEvent
import java.awt.event.ActionListener
import algo.Generator1
import javax.swing.JButton
import javax.swing.JLabel
import javax.swing.JPanel
import measures.GraphMeasures
import utils.Icon
import gui.components.SpinSlider
import guidyn.components.ProgressBar

class MeasurePanel extends JPanel(new BorderLayout) {
    // val topPanel = new JPanel(new FlowLayout(FlowLayout.LEFT))
    // val topPanel = new JPanel(new GridLayout(2, 0))
    val topPanel = new JPanel(new GridLayout(0, 1))
    
    
    val clusteringCoefficient = new ButtonField("Avg. Clust. Coeff.", () => m.clusteringCoefficient)
    val randomClusteringCoefficient = new ButtonField("Rand. Clust. Coeff.", () => m.randomClusteringCoefficient)
    val avgDegree = new ButtonField("Avg. Degree", () => m.avgDegree)
    val avgShortestPath = new ButtonField("Avg. Short. Path", () => m.avgShortestPath)
    val withinInertia = new ButtonField("Within Inertia", () => m.withinInertiaRatio)
//    val betweenInertia = new ButtonField("Between Inertia", () => m.betweenInertiaRatio)
    val observedEdgesHomophily = new ButtonField("Observed Homophily", () => m.homophilyObserved)
    val expectedEdgesHomophily = new ButtonField("Expected Homophily", () => m.homophilyExpected)
    val diameter = new ButtonField("Diameter", () => m.diameter)
    val edgesIntra = new ButtonField("Edges Btw.", () => { m.nbInternalExternalEdges._2 })
    val edgesInter = new ButtonField("Edges Wth.", () => { m.nbInternalExternalEdges._1 })
//    val concentrationRelative = new ButtonField("Concentrat. Rel.", () => m.relativeConcentrationWithin)
    val nbEdgesLabel = new ButtonField("Nb Edges", () => m.nbEdges)
    val modularity = new ButtonField("Modularity", () => m.modularity)
    
//    val threshold = new SpinSlider("Minimum Similarity:", 0, 1000, 3, 1)
    
    
    val measures = List(
          observedEdgesHomophily,
          expectedEdgesHomophily,
          withinInertia,
          modularity,
          clusteringCoefficient,
          randomClusteringCoefficient,
          avgDegree,
          avgShortestPath,
          
//          betweenInertia,
          diameter,
          edgesInter,
          edgesIntra,
//            concentrationRelative,
          nbEdgesLabel
    )
    
    val fastMeasures = measures.filter(m => m != diameter && m != avgShortestPath)
    
    val run = new JButton("Compute all")
    run.addActionListener(new ActionListener { def actionPerformed(e : ActionEvent) = { updateMeasures() } })
    def updateMeasures() { for(m <- measures) m.doUpdate() }
    def updateFastMeasures() { for(m <- fastMeasures) m.doUpdate() }
    def resetMeasures() { for(m <- measures) m.resetValue() }
    
    topPanel.add(run)
//    topPanel.add(threshold)
    for(c <- measures) topPanel.add(c)
    
    add(topPanel, BorderLayout.NORTH)
    topPanel.setBackground(Color.white)
    setBackground(Color.white)
    
    var m : GraphMeasures = _
    
    def setGenerator(gen : Generator1) {
        m = new GraphMeasures(gen.graph, gen.dev)
    }
}

class ButtonField(label : String, f : () => Double = () => 35D) extends JPanel(new BorderLayout()) {
    val labelField = new JLabel(label + ":")
    val valueField = new JLabel("?   ")
    val updateButton = new JButton(Icon.get("arrow_rotate_clockwise"))
    setBackground(Color.white)
    add(valueField)
    
    val leftPanel = new JPanel(new FlowLayout(FlowLayout.LEFT))
    leftPanel.setBackground(Color.white)
    leftPanel.add(updateButton)
    leftPanel.add(labelField)
    add(leftPanel, BorderLayout.WEST)
    add(valueField, BorderLayout.EAST)
    
    def resetValue() = valueField.setText("?   ")
    
    updateButton.addActionListener( new ActionListener { def actionPerformed(e : ActionEvent) { doUpdate } })
    
    def doUpdate() = {
        ProgressBar.startTask("Computing measure " + label)
        setValue(f())
    }
    
    def setValue(value : Double) = {
        if(math.floor(value) == value) {
            valueField.setText(s"${value.toInt}   ")
        } else {
            valueField.setText(f"${value}%.2f   ")
        }
         
    }
    
    def setValue(value : String) = { valueField.setText(value) }
}