/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package gui

import java.awt.Component
import java.io.DataInputStream
import java.io.DataOutputStream
import java.io.File
import java.io.FileInputStream
import java.io.FileOutputStream
import scala.collection.mutable.ListBuffer
import bibliothek.extension.gui.dock.theme.EclipseTheme
import bibliothek.gui.DockFrontend
import bibliothek.gui.dock.DefaultDockable
import bibliothek.gui.dock.SplitDockStation
import bibliothek.util.xml.XIO
import java.io.FileReader
import bibliothek.util.xml.XElement
import java.io.FileWriter
import utils.Icon
import gui.components.Savable
import java.io.IOException
import bibliothek.gui.dock.station.split.SplitDockGrid
import scala.collection.JavaConversions._

object ViewManager {
	var station = new SplitDockStation()
	var savableComponents = new ListBuffer[Savable]
	
	val layoutFile = ".layout"
	// val hideImage = Icon.get("application_delete")
	// val showImage = Icon.get("application_add")
	    
	var frontEnd = new DockFrontend()
	frontEnd.getController().setTheme(new EclipseTheme())
	frontEnd.addRoot("split", station)
	
	def registerPanel(component : Component, title : String, icon : Option[String] = None) = {
		val dockable = new DefaultDockable()
		dockable.add(component)
		dockable.setTitleText(title)
		dockable.setTitleIcon(Icon.get(icon.getOrElse("tux")))
		frontEnd.addDockable(title, dockable)
		station.drop(dockable)
	}
	
	def registerSavable(savable : Savable) = savableComponents += savable
	
	def loadLayout() {
	    val grid = new SplitDockGrid()
	    for(d <- frontEnd.getDockables()) {
	        d.getTitleText() match {
	            case "Parameters" => grid.addDockable( 0, 0, 280, 400, d)
	            case "GraphLayout" => grid.addDockable( 280, 0, 500, 400, d)
	            case "Measures" => grid.addDockable( 780, 0, 200, 400, d)
	        }
	    }
        station.dropTree( grid.toTree() );
		if(new File(layoutFile).exists()) { frontEnd.readXML(XIO.read(new FileReader(layoutFile))) }
		savableComponents.foreach(_.load())
	}
	
	def saveLayout() {
	    try {
	        val xroot = new XElement("layout")
	        frontEnd.writeXML(xroot)
	        val fw = new FileWriter(layoutFile)
		    fw.write(xroot.toString())
		    fw.flush()
		    fw.close()
	    } catch {
	    	case e : IOException => println("Unable to save the layout. Check that the directory has write access")
	    }
	    

		savableComponents.foreach(_.save())
	}
}