/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package gui.components

import javax.swing.JPanel
import java.awt.BorderLayout
import org.jfree.data.category.DefaultCategoryDataset
import org.jfree.data.xy.DefaultXYDataset
import org.jfree.chart.ChartFactory
import org.jfree.chart.ChartPanel
import javax.swing.JFrame
import java.awt.Color
import org.jfree.chart.plot.XYPlot
import org.jfree.chart.axis.LogarithmicAxis
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer
import java.awt.BasicStroke
import org.jfree.util.ShapeUtilities
import java.awt.geom.Ellipse2D
import org.jfree.chart.axis.NumberAxis
import utils.SwingEvents._
import scala.swing.event.MouseEntered
import java.awt.event.MouseEvent
import java.awt.geom.Rectangle2D
import java.awt.Rectangle
import org.w3c.dom.Document
import java.io.OutputStream
import org.jfree.data.function.LineFunction2D
import org.jfree.data.function.PowerFunction2D
import org.jfree.data.general.DatasetUtilities
import algo.Generator1
import measures.GraphMeasures

class DegreeDistribution(gen : Option[Generator1] = None, distribution: Array[(Int, Int)] = Array[(Int, Int)]((1, 1)),
    isLogarithmic: Boolean = true) extends JPanel(new BorderLayout()) {
    val dataset = new DefaultXYDataset()
    setDistribution(distribution, gen)
    val chart = ChartFactory.createXYLineChart("", "", "", dataset)
    chart.setBackgroundPaint(Color.white)
    val plot = chart.getXYPlot()
    chart.removeLegend()

    // Serie renderer
    val renderer = new XYLineAndShapeRenderer()
    // renderer.setPaint(Color.darkGray)
    renderer.setSeriesShape(0, new Ellipse2D.Double(-3D, -3D, 6, 6))
    renderer.setSeriesStroke(0, new BasicStroke(2))
    renderer.setSeriesPaint(0, Color.darkGray)
    renderer.setSeriesShapesVisible(1, false)
    renderer.setSeriesStroke(1, new BasicStroke(1))
    renderer.setSeriesPaint(1, Color.red)
    plot.setRenderer(0, renderer)
    plot.setRenderer(1, renderer)

    // Axis rendering
    setLogarithmic(isLogarithmic)

    plot.setBackgroundPaint(Color.white)
    plot.setRangeGridlinePaint(Color.lightGray)
    plot.setDomainGridlinePaint(Color.lightGray)

    val chartPanel = new ChartPanel(chart)
    chartPanel.setBackground(Color.white)
    chartPanel.setDisplayToolTips(true)
    add(chartPanel)

    chartPanel.onMouseWheelMove { e =>
        val zoomPoint = e.getPoint()
        if (e.getWheelRotation() > 0) {
            chartPanel.zoomInBoth(zoomPoint.getX(), zoomPoint.getY());
        } else if (e.getWheelRotation() < 0) {
            chartPanel.zoomOutBoth(zoomPoint.getX(), zoomPoint.getY());
        }
    }
    

    chartPanel.onClick { e => if (e.getButton() == MouseEvent.BUTTON2) { chartPanel.restoreAutoBounds() } }

    def setDistribution(distribution: Array[(Int, Int)], generator : Option[Generator1]): Unit = {
        val maxDegree = 15
        for(gen <- generator) {
            val dataX = new Array[Double](distribution.length)
	        val dataY = new Array[Double](distribution.length)
	        for (((x, y), i) <- distribution.zipWithIndex) {
	            dataX(i) = if (x == 0) 0.0001 else x
	            // dataY(i) = if (y == 0) 0.0001 else y
	            dataY(i) = y.toDouble / gen.nbVertices
	        }
	        dataset.addSeries(0, Array(dataX, dataY))
	        val measures = new GraphMeasures(gen.graph, gen.dev)
//	        val gamma = measures.meanGammaScaleFree(15)
//	        println("gamma: " + gamma)
//	        val dataXFit = (1.5D to maxDegree by 0.001).toArray
//	        val dataYFit = dataXFit.map(x => math.pow(x, -gamma))
//	        dataset.addSeries(1, Array(dataXFit, dataYFit))
        }
    }

    def setLogarithmic(isLogarithmic: Boolean) = {
        if (isLogarithmic) {
            plot.setDomainAxis(new LogarithmicAxis("Degree"))
            plot.setRangeAxis(new LogarithmicAxis("Frequency"))
        } else {
            plot.setDomainAxis(new NumberAxis("Degree"))
            plot.setRangeAxis(new NumberAxis("Frequency"))
        }
    }
    
    def outputPdf(fileName : String) = {
        utils.PdfOutput.createPdf(this, fileName, Option(getSize()))
    }
}

object TestChart extends JFrame("Test JChart") {

    def main(args: Array[String]) = {
        setSize(800, 600)
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE)
//        val dd = new DegreeDistribution(Array((1, 45), (2, 32)))
//        add(dd)
//        dd.setDistribution(Array((1, 15), (2, 45)))
        setVisible(true)
    }

}