/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package gui.components

import javax.swing.JPanel
import java.awt.Graphics
import java.awt.Graphics2D
import javax.swing.JFrame
import java.awt.BorderLayout
import java.awt.Color
import java.awt.RenderingHints
import java.awt.Dimension
import scala.Array.canBuildFrom

class DegreeDistributionOld(var distribution : Array[(Int, Int)] = Array[(Int, Int)]((0, 0)), defaultSize : Option[Dimension] = None) extends JPanel {
	var maxDegree = distribution.map(_._1).max
	var maxFreq = distribution.map(_._2).max
	
	val xLegend = "Degree"
    val yLegend = "Freq"
	
    val borderRight = 20
    val borderLeft = 40
    val borderTop = 20
    val borderBottom = 40
    
    def setDistribution(distribution : Array[(Int, Int)]) {
	    this.distribution = distribution
	    maxDegree = distribution.map(_._1).max
	    maxFreq = distribution.map(_._2).max
	}
    
	override def paint(g : Graphics) {
	    val g2d = g.asInstanceOf[Graphics2D]
	    val width = getWidth()
	    val height = getHeight()
	    g2d.setColor(Color.white)
        g2d.fillRect(0, 0, width, height)
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g2d.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
        
        g2d.setColor(Color.black)
        
	    val xLegendWidth = g2d.getFontMetrics().stringWidth(xLegend)
		val yLegendWidth = g2d.getFontMetrics().stringWidth(yLegend)
	    
	    val orig = g2d.getTransform()
		// g2d.rotate(-Math.PI/2)
	    g2d.rotate(-Math.PI/2)
		g2d.drawString("Freq", -(height - yLegendWidth) / 2, 15)
		g2d.setTransform(orig)
		
		g2d.drawString("Degree", (width - xLegendWidth) / 2, height - 10)
		g2d.drawString("0", borderLeft - 4, height - borderBottom + 17)
		g2d.drawString("0", borderLeft - 15, height - borderBottom + 4)
		
		val maxFreqTxtLength = g2d.getFontMetrics().stringWidth(maxFreq.toString)
		g2d.drawString(maxFreq.toString, borderLeft - maxFreqTxtLength - 6, borderTop + 5)
		
		val maxDegreeTxtLength = g2d.getFontMetrics().stringWidth(maxDegree.toString)
		g2d.drawString(maxDegree.toString, width - borderRight - (maxDegreeTxtLength / 2), height - borderBottom + 17)
		
		g2d.drawLine(borderLeft, borderTop, borderLeft, height - borderBottom + 4)
		g2d.drawLine(borderLeft - 4, height - borderBottom, width - borderRight, height - borderBottom)
		g2d.drawLine(width - borderRight, height - borderBottom, width - borderRight, height - borderBottom + 4)
		g2d.drawLine(borderLeft - 4, borderTop, borderLeft, borderTop)
		
		val xLength = getWidth() - borderLeft - borderRight
	    val yLength = getHeight() - borderTop - borderBottom
		
		val coord = distribution.map(a => normalize(a._1, a._2))
		for(i <- 0 until coord.length - 1) { g2d.drawLine(coord(i)._1, coord(i)._2, coord(i + 1)._1, coord(i + 1)._2) }
        val radius = 3
		for((x, y) <- coord) {
		    g2d.setColor(Color.white)
		    g2d.fillOval(x - radius, y - radius, radius * 2 + 1, radius * 2 + 1)
		    g2d.setColor(Color.black)
		    g2d.drawOval(x - radius, y - radius, radius * 2 + 1, radius * 2 + 1)
	    }
	    
	}
	
	def normalize(x : Int, y : Int) : (Int, Int) = {
	    val xLength = getWidth() - borderLeft - borderRight
	    val yLength = getHeight() - borderTop - borderBottom
	    val px = (x.toDouble / maxDegree) * xLength + borderLeft
	    val py = getHeight() - borderBottom - (y.toDouble / maxFreq) * yLength
	    (px.toInt, py.toInt)
	}
	
	override def getPreferredSize() = {
	    new Dimension(getWidth(), getWidth())
	}
	
	override def getWidth = { if(defaultSize.isDefined) { defaultSize.get.width } else { super.getWidth } }
	override def getHeight = { if(defaultSize.isDefined) { defaultSize.get.height } else { super.getHeight } }
}


object Main2 extends JFrame {
    
    def main(args: Array[String]): Unit = {
        createAndShowGUI()
    }
    
    def createAndShowGUI() {
//        setSize(800, 600)
//        setLayout(new BorderLayout())
//        setTitle("Graph Generator")
//        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE)
//        val distrib = Array((1, 450), (2, 23), (8, 10), (24, 2), (56, 1))
//        add(new DegreeDistribution(distrib), BorderLayout.CENTER)
//        setVisible(true)
    }
}