/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package gui.components

import java.util.prefs.Preferences
import java.io.IOException

trait Savable {
	def save()
    def load()
	
	SavableManager.registerSavable(this)
}

object SavableManager {
    val pref = Preferences.userRoot().node("GraphGen")
    
    var savables = List[Savable]()
    def registerSavable(savable : Savable) { savables = savable :: savables }
    
    def loadAll() = { for(s <- savables) { s.load } }
    def saveAll() = { for(s <- savables) {
        try {
            s.save
        } catch {
            case e : IOException => { println("Unable to save state") }
        }
    } }
    
    def save[T](label : String, value : T) = {
        value match {
            case e : Int => pref.putInt(label, e)
            case e : Double => pref.putDouble(label, e)
            case e : Boolean => pref.putBoolean(label, e)
            case e : String => pref.put(label, e)
            case e : Float => pref.putFloat(label, e)
            case e : Long => pref.putLong(label, e)
            case e : Array[Byte] => pref.putByteArray(label, e)
            case _ => pref.put(label, value.toString)
        }
    }

    def load[T](label : String, default : T) : T = {
        default match {
            case e : Int => pref.getInt(label, e).asInstanceOf[T]
            case e : Double => pref.getDouble(label, e).asInstanceOf[T]
            case e : Boolean => pref.getBoolean(label, e).asInstanceOf[T]
            case e : String => pref.get(label, e).asInstanceOf[T]
            case e : Float => pref.getFloat(label, e).asInstanceOf[T]
            case e : Long => pref.getLong(label, e).asInstanceOf[T]
            case e : Array[Byte] => pref.getByteArray(label, e).asInstanceOf[T]
            case _ => pref.get(label, default.toString).asInstanceOf[T]
        }
    }
}