/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package gui.components

import java.awt.BorderLayout
import java.awt.FlowLayout
import com.alee.laf.slider.WebSlider
import gui.Main
import javax.swing.Box
import javax.swing.JLabel
import javax.swing.JPanel
import javax.swing.JSpinner
import javax.swing.SpinnerNumberModel
import javax.swing.event.ChangeListener
import javax.swing.event.ChangeEvent
import com.alee.extended.label.WebHotkeyLabel
import javax.swing.BoxLayout
import com.alee.managers.tooltip.TooltipManager
import com.alee.managers.tooltip.TooltipWay
import utils.SwingEvents._
import java.awt.Dimension

case class SliderDescriptor(min : Int, max : Int, default : Int, step : Int, sliderMapping : Int => Double) {}

class SpinSlider(val label: String, min: Double, max: Double, val default: Double, step: Double = 1, help : String = "") extends JPanel(new BorderLayout()) with Savable {
    
    // Build the components
    val name = new WebHotkeyLabel(label)
    if(!help.isEmpty()) TooltipManager.addTooltip (name, help, TooltipWay.down, 0)
    val model = new SpinnerNumberModel(default, min, max, step)
    val spinner = new JSpinner(model)
    
    if(max == 1D) {
//        spinner.setMinimumSize(new Dimension(200, 50))
        spinner.setPreferredSize(new Dimension(70, 27))
        
    }
    
    def setEditable(isEnabled : Boolean) = {
        spinner.setEnabled(isEnabled)
    }
    
    def setValue(v: Double) = { model.setValue(v) }

    def value : Double = model.getValue().asInstanceOf[Double]
    
    // Layout the components
//    val top = new JPanel()
//    val layoutTop = new BoxLayout(top, BoxLayout.LINE_AXIS)
//    top.setLayout(layoutTop)
//    top.add(name)
//    top.add(Box.createHorizontalGlue())
//    top.add(spinner)
//    add(top, BorderLayout.CENTER)
    add(name, BorderLayout.WEST)
    add(spinner, BorderLayout.EAST)
    
    def onChange(f : ChangeEvent => Unit) = {
        spinner.addChangeListener(new ChangeListener() {
            def stateChanged(e : ChangeEvent) = { f(e) }
        })
    }
    
    // Save state
    def save() = { SavableManager.save(label, value) }
    def load() = { 
        setValue(SavableManager.load(label, default)) 
//        val sliderDefault = if(sliderDescriptor.isDefined) sliderDescriptor.get.default else default.toInt
//        slider.setValue(SavableManager.load(label + "Slider", sliderDefault))
    }
}

