/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package gui.graph

import edu.uci.ics.jung.visualization.control.TranslatingGraphMousePlugin
import edu.uci.ics.jung.visualization.control.RotatingGraphMousePlugin
import edu.uci.ics.jung.visualization.control.ShearingGraphMousePlugin
import edu.uci.ics.jung.visualization.control.DefaultModalGraphMouse
import java.awt.Paint
import algo.Generator1
import java.awt.Dimension
import edu.uci.ics.jung.graph.UndirectedSparseGraph
import edu.uci.ics.jung.visualization.GraphZoomScrollPane
import java.awt.Color
import org.apache.commons.collections15.Transformer
import edu.uci.ics.jung.visualization.VisualizationViewer
import utils.Colors
import java.awt.event.MouseAdapter
import edu.uci.ics.jung.visualization.decorators.EdgeShape
import edu.uci.ics.jung.visualization.control.ModalGraphMouse
import java.awt.event.InputEvent
import java.awt.event.MouseEvent
import javax.swing.JPanel
import java.awt.BorderLayout
import scala.collection.JavaConversions._
import gui.RunPanel
import org.apache.commons.collections15.Predicate
import edu.uci.ics.jung.graph.util.Context
import edu.uci.ics.jung.graph.Graph
import algo.Generator1
import java.awt.GradientPaint
import java.awt.Stroke
import java.awt.BasicStroke
import scala.collection.mutable.HashSet
import java.awt.RadialGradientPaint
import java.awt.geom.Point2D
import java.awt.MultipleGradientPaint.CycleMethod

object GraphView extends JPanel(new BorderLayout()) {
    var viewer: VisualizationViewer[Int, Int] = _
	var jungGraph : UndirectedSparseGraph[Int, Int] = _
	var communitiesGraph = new Array[UndirectedSparseGraph[Int, Int]](0)

	def setGraph(gen: Generator1) = {
        // Build the jung graph
        jungGraph = new UndirectedSparseGraph[Int, Int]()
        for (v <- 0 until gen.nbVertices) { jungGraph.addVertex(v) }
        var edgeId = 0
        for (v1 <- 0 until gen.nbVertices; v2 <- gen.graph(v1)) {
            jungGraph.addEdge(edgeId, v1, v2)
            edgeId += 1
        }
        
        // Build the jung graph for each community
        communitiesGraph = for(community <- gen.graph.partition.values.toArray) yield {
            val communityGraph = new UndirectedSparseGraph[Int, Int]()
            for (v <- community) {
	            communityGraph.addVertex(v)
	            for(e <- jungGraph.getIncidentEdges(v)) {
	                val endPoints = jungGraph.getEndpoints(e)
	                if(community.containsAll(endPoints)) { communityGraph.addEdge(e, endPoints.getFirst(), endPoints.getSecond()) }
	            }
            }
            communityGraph
        }
        
        val vertexPaint = new Transformer[Int, Paint]() {
            val palette = Colors.getMatchingColors(gen.k)
            val palette2 = Colors.getMatchingColors(gen.k, 0.4f, 0.9f)
            def transform(v: Int): Paint = {
                if (viewer.getPickedVertexState().getPicked().contains(v)) { Color.red } else {
                    if(GraphOptions.highlightBetweenEdges.isSelected()) {
                        val curCommunity = gen.graph.vertexCommunities(v)
                        val colors = HashSet[Color]()
                        for(n <- jungGraph.getNeighbors(v)) { colors.add(palette(gen.graph.vertexCommunities(n))) }
                        val nbColors = colors.size
                        
                        if(nbColors == 1) {
                            palette(gen.graph.vertexCommunities(v))
                        } else {
                            val pt = viewer.getGraphLayout().transform(v)
                            val dist = (for(i <- 0 until nbColors) yield { 0.1f + 0.9f * (i.toFloat / nbColors)}).toArray
                    		new RadialGradientPaint(pt, 10, dist, colors.toArray, 
                                 CycleMethod.REPEAT)
                        }
                        
                        
//                        if(jungGraph.getNeighbors(v).exists(n => gen.vertexCommunities(n) != curCommunity)) {
//                            palette2(gen.vertexCommunities(v))
//                        } else {
//                            palette(gen.vertexCommunities(v))
//                        }
                    } else {
                        palette(gen.graph.vertexCommunities(v))
                    }
                     
                }
            }
        }

        val gm = new DefaultModalGraphMouse[Int, Int]()
        gm.setMode(ModalGraphMouse.Mode.PICKING)
        val translatingPlugin = new TranslatingGraphMousePlugin(InputEvent.BUTTON3_MASK)

        val rotatingPlugin = new RotatingGraphMousePlugin()
        val shearingPlugin = new ShearingGraphMousePlugin()
        gm.add(rotatingPlugin)
        gm.add(shearingPlugin)
        gm.add(translatingPlugin)

        val dim = new Dimension(getWidth(), getHeight())
        viewer = new VisualizationViewer(GraphOptions.getSelectedLayout(), dim)
        viewer.setGraphMouse(gm)
        viewer.setBackground(Color.white)
        viewer.getRenderContext().setVertexFillPaintTransformer(vertexPaint)
        viewer.getRenderContext().setEdgeShapeTransformer(new EdgeShape.Line())
        viewer.getRenderContext().setVertexIncludePredicate(new VertexDisplayPredicate())
        viewer.getRenderContext().setEdgeIncludePredicate(new EdgeDisplayPredicate())
//        viewer.getRenderContext().setEdgeFillPaintTransformer(new EdgePaintTransformer(gen))
//        viewer.getRenderContext().setEdgeDrawPaintTransformer(new EdgePaintTransformer(gen))
        viewer.getRenderContext().setEdgeStrokeTransformer(new EdgeStrokeTransformer)
        
        viewer.addMouseListener(new MouseAdapter() {
            override def mouseClicked(e: MouseEvent) {
                if (e.getButton() == MouseEvent.BUTTON1 && e.getClickCount() >= 2) {
                    for (v <- viewer.getPickedVertexState().getPicked()) {
                        val curCom = RunPanel.gen.graph.vertexCommunities(v)
                        for (v2 <- 0 until RunPanel.gen.nbVertices) {
                            if (RunPanel.gen.graph.vertexCommunities(v2) == curCom) {
                                viewer.getPickedVertexState().pick(v2, true)
                            }
                        }
                    }
                }
            }
        })
        
        // Add the steps
//        GraphOptions.step.setStepsAmount(gen.step)
//        GraphOptions.step.setSelectedStep(gen.step)
        GraphOptions.step.setShowLabels(false)
        
        val zoomScroll = new GraphZoomScrollPane(viewer)
        removeAll()
        add(zoomScroll, BorderLayout.CENTER);
        repaint()
    }
    
    class VertexDisplayPredicate[V, E] extends Predicate[Context[Graph[V, E], V]] {
		def evaluate(context: Context[Graph[V, E], V]) : Boolean = {
			val curStep = GraphOptions.step.getSelectedStep()
//			val vStep = RunPanel.gen.verticesStep(context.element.asInstanceOf[Int])
//			(vStep == curStep) || (!GraphOptions.onlyCurrentStep.isSelected() && vStep <= curStep)
			true
		}
	}
    
    class EdgeDisplayPredicate[V, E] extends Predicate[Context[Graph[V, E], V]] {
		def evaluate(context: Context[Graph[V, E], V]) : Boolean = {
		    GraphOptions.toggleEdgeDisplay.isSelected()
		}
	}
    
    /*
    class EdgePaintTransformer[V, E](gen : Generator1) extends Transformer[E, Paint] {
		def transform(edge : E) : Paint = {
		    val palette = Colors.getMatchingColors(gen.k)
		    val endPoints = jungGraph.getEndpoints(edge.asInstanceOf[Int])
		    val (v1, v2) = (endPoints.getFirst(), endPoints.getSecond())
		    val (p1, p2) = (viewer.getGraphLayout().transform(v1), viewer.getGraphLayout().transform(v2))
		    val color1 = palette(gen.vertexCommunities(endPoints.getFirst()))
		    val color2 = palette(gen.vertexCommunities(endPoints.getSecond()))
		    new GradientPaint(p1, color1, p2, color2)
		}
	}
	*/
    
    class EdgeStrokeTransformer[V, E] extends Transformer[E, Stroke] {
        val stroke = new BasicStroke(2f)
		def transform(edge : E) = stroke
	}
    
}