/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package gui.graph

import edu.uci.ics.jung.algorithms.layout.Layout
import edu.uci.ics.jung.graph.Graph
import java.awt.Dimension
import java.awt.geom.Point2D
import algo.Generator1
import gui.RunPanel

class PlotLayout(var graph : Graph[Int,Int]) extends Layout[Int,Int] {
    var border : Double = 30
    var isFirst = true
    val dimension = new Dimension(0, 0)
    val minAttrs = new Array[Double](2)
    val maxAttrs = new Array[Double](2)
    var attributes = RunPanel.gen.graph.attr
    initGraph
    
    def getGraph() = graph
    
    def getSize() = {
        if(isFirst) {
            isFirst = false
            null
        } else {
            dimension
        }
    }
    
    def initialize() = {}
    
    def isLocked(v : Int) = true
    
    def lock(v : Int, state: Boolean) = {}
    
    def reset() = {}
    
    
    def setGraph(g : Graph[Int,Int]) {
        attributes = RunPanel.gen.graph.attr
        graph = g
        initGraph
    }
    
    def initGraph = {
        if(attributes(0).size >= 2) {
            minAttrs(0) = attributes.values.map(_(0)).min
	        minAttrs(1) = attributes.values.map(_(1)).min
	        maxAttrs(0) = attributes.values.map(_(0)).max
	        maxAttrs(1) = attributes.values.map(_(1)).max
        }
    }

    def getXY(v2 : Int) : Point2D = {
        if(attributes(0).size >= 2) {
            val v = v2.toString.toInt
	        val w = dimension.getWidth()
	        val h = dimension.getHeight()
	        val xn = (attributes(v)(0) - minAttrs(0)) / (maxAttrs(0) - minAttrs(0))
	        val yn = (attributes(v)(1) - minAttrs(1)) / (maxAttrs(1) - minAttrs(1))
	        val x = (xn * (w - (2 * border)) + border)
	        val y = (yn * (h - (2 * border)) + border)
	        new Point2D.Double(x, y)
        } else {
            new Point2D.Double(0, 0)
        }
    }
    
    def setInitializer(initializer: org.apache.commons.collections15.Transformer[Int,Point2D]) = { }
    
    def setLocation(v : Int, location : Point2D) = location.setLocation(getXY(v))
    
    def setSize(d : Dimension) = {
        dimension.setSize(d.getWidth() - 30, d.getHeight() - 30)
    }
    
    def transform(v2 : Int) : Point2D = {
        getXY(v2)
    }
}