/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package guidyn

import java.awt.BorderLayout
import java.awt.Color
import java.awt.event.WindowAdapter
import java.awt.event.WindowEvent
import scala.collection.mutable.ArrayBuffer
import gui.components.Savable
import gui.components.SavableManager
import guidyn.components.CommunityDynamics
import guidyn.components.HandScrollListener
import guidyn.components.Measures
import guidyn.components.Parameters
import guidyn.components.RunPanel
import guidyn.components.graph.GraphPanel
import guidyn.components.listeners.GraphUpdateListener
import guidyn.components.listeners.TimestampUpdateListener
import javax.swing.JDialog
import javax.swing.JFrame
import javax.swing.JScrollPane
import javax.swing.UIManager
import utils.Icon
import guidyn.components.DegreeDistributions

object Main extends JFrame with Savable {
    setIconImage(Icon.get("favicon").getImage())
    
    val graphUpdateListeners = ArrayBuffer[GraphUpdateListener]()
    def addGraphUpdateListeners(e : GraphUpdateListener) { graphUpdateListeners.append(e) }
    
    val communityDynamics = new CommunityDynamics()
    
    val timestampUpdateListeners = ArrayBuffer[TimestampUpdateListener]()
    def addTimestampUpdateListeners(e : TimestampUpdateListener) { timestampUpdateListeners.append(e) }
    
    def main(args: Array[String]): Unit = {
        com.alee.laf.WebLookAndFeel.install()
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            def run() {
                UIManager.put("ToolTip.background", Color.black)
                JFrame.setDefaultLookAndFeelDecorated(true)
                JDialog.setDefaultLookAndFeelDecorated(true)
                createAndShowGUI()
            }
        })
    }
    
    def save() = {
        SavableManager.save("frameWidth", this.getWidth())
        SavableManager.save("frameHeight", this.getHeight())
        SavableManager.save("frameX", this.getX())
        SavableManager.save("frameY", this.getY())
    }
    
    def load() = {
        setLocation(SavableManager.load("frameX", 0), SavableManager.load("frameY", 0))
        setSize(SavableManager.load("frameWidth", 800), SavableManager.load("frameHeight", 600))
    }
    
    def createAndShowGUI() {
        setLayout(new BorderLayout())
        setTitle("Dynamic Graph Generator")
        
        addWindowListener(new WindowAdapter { override def windowClosing(e : WindowEvent) = { exit } })
        setVisible(true)
        
        var rootPanel = getContentPane()
	    rootPanel.setLayout(new BorderLayout())
	    rootPanel.add(ViewManager.station, BorderLayout.CENTER)
	    rootPanel.add(gui.panels.StatusBar, BorderLayout.SOUTH)
	    rootPanel.add(RunPanel, BorderLayout.NORTH)
        
	    ViewManager.registerPanel(GraphPanel, "Graph", Option("chart_organisation"))
	    
	    val communityDynamicsScrollPane = new JScrollPane(communityDynamics)
        xx = communityDynamicsScrollPane
	    val scrollListener = new HandScrollListener()
        communityDynamics.addMouseMotionListener(scrollListener)
        communityDynamics.addMouseListener(scrollListener)
	    
	    ViewManager.registerPanel(communityDynamicsScrollPane, "Community Dynamics", Option("control_equalizer"))
	    ViewManager.registerPanel(new JScrollPane(Parameters), "Parameters", Option("wrench"))
	    ViewManager.registerPanel(Measures, "Measures", Option("chart_curve"))
	    ViewManager.registerPanel(DegreeDistributions, "Degree Distributions", Option("chart_curve"))
	    ViewManager.loadLayout()
	    SavableManager.loadAll
    }
    
    var xx : JScrollPane = _
    
    def exit() = {
        SavableManager.saveAll
        ViewManager.saveLayout()
        System.exit(0)
    }
}

