/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package guidyn.components

import java.awt.BorderLayout
import java.awt.Component
import java.awt.Dimension
import com.alee.laf.button.WebButton
import gui.components.Savable
import gui.components.SavableManager
import gui.components.TitledSeparator
import javax.swing.BoxLayout
import javax.swing.JPanel
import utils.Icon
import utils.SwingEvents.convertComponent
import javax.swing.Timer
import java.awt.event.ActionListener
import java.awt.event.ActionEvent

class CollapsablePanel(label: String) extends JPanel(new BorderLayout()) with Savable with ActionListener {
    // val iconDown = Icon.get("bullet_arrow_down")
    val iconDown = Icon.get("chevron-down-16")
    // val iconUp = Icon.get("bullet_arrow_up")
    val iconUp = Icon.get("chevron-up-16")
    val header = new JPanel(new BorderLayout())
    header.add(new TitledSeparator(label), BorderLayout.CENTER)
    val collapseButton = new WebButton(iconUp)
    collapseButton.setUndecorated(true)
    header.onClick { e => setCollapsed(!componentPanel.isVisible()) }
    collapseButton.onClick { e => setCollapsed(!componentPanel.isVisible()) }
    
    var from = 0
    var to = 0
    var delta = 10
    var maxHeight = 0
    
    def actionPerformed(e: ActionEvent) {
        if(to == 0) {
            curHeight -= 10
            curHeight = math.max(0, curHeight)
            if(curHeight == 0) {
                myTimer.stop()
                componentPanel.setVisible(false)
            }
        } else {
            curHeight += 10
            curHeight = math.min(curHeight, maxHeight)
            if(curHeight == maxHeight) {
                myTimer.stop()
                
            }
        }
        componentPanel.setPreferredSize(new Dimension(0, curHeight))
        revalidate()
    }
    val myTimer = new Timer(10, this)
    def setCollapsed(isCollapsed: Boolean) = {
//        maxHeight = math.max(maxHeight, componentPanel.getHeight())
        collapseButton.setIcon((if (!isCollapsed) iconDown else iconUp))
        componentPanel.setVisible(true)
        maxHeight = math.max(maxHeight, componentPanel.getHeight())
        if(isCollapsed == false) {
//            println("Collapse")
            from = maxHeight - 30
            to = 0
        } else {
//            println("Expand")
            from = 0
            to = maxHeight - 30
        }
        curHeight = from
        myTimer.start()
    }

    header.add(collapseButton, BorderLayout.EAST)
    add(header, BorderLayout.NORTH)

    val componentPanel = new JPanel()
    componentPanel.setLayout(new BoxLayout(componentPanel, BoxLayout.Y_AXIS))
    add(componentPanel, BorderLayout.CENTER)

    def addComponent(component: Component) {
        componentPanel.add(component)
    }

    var curHeight = 0
    
    
    def save() = { SavableManager.save("isCollapsed." + label, componentPanel.isVisible()) }
    def load() = { }
}