/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package guidyn.components

import gui.components.Savable
import javax.swing.JPanel
import java.awt.BorderLayout
import guidyn.components.listeners.GraphUpdateListener
import java.awt.FlowLayout
import algo.structures.DynamicGraph
import scala.collection.mutable.HashMap
import java.awt.Color
import javax.swing.JScrollPane
import gui.components.RoundedSwitch
import javax.swing.JLabel
import utils.SwingEvents._

object DegreeDistributions extends JPanel(new BorderLayout()) with GraphUpdateListener {
    val plotWidth = 300
    
    val logScalePanel = new JPanel(new FlowLayout(FlowLayout.LEADING))
    val switchLogScaleButton = new RoundedSwitch()
    switchLogScaleButton.onAction { e =>
        plotsContainer.getComponents().map(_.asInstanceOf[Plot].setLogarithmic(switchLogScaleButton.isSelected())) 
    }
    logScalePanel.add(switchLogScaleButton)
    logScalePanel.add(new JLabel("Use Logarithmic Scale"))
    
    val plotsContainer = new JPanel(new FlowLayout(FlowLayout.LEADING))
    add(new JScrollPane(plotsContainer), BorderLayout.CENTER)
    add(logScalePanel, BorderLayout.NORTH)
    
    plotsContainer.setBackground(Color.white)
    setBackground(Color.white)
    
    def graphUpdated(g: DynamicGraph) = {
        plotsContainer.removeAll()
        for((graph, t) <- g.graphs.zipWithIndex) {
            val degreeDistribution = HashMap[Int, Int]()
            for(v <- graph.vertices) { degreeDistribution(graph(v).size) = degreeDistribution.getOrElse(graph(v).size, 0) + 1 }
            val degreeDistributionArray = degreeDistribution.toArray.sortBy(_._1)
            val freq = degreeDistributionArray.map(x => x._2.toDouble / graph.nbVertices)
            val degrees = degreeDistributionArray.map(_._1.toDouble).map(x => math.max(x, 1))
            
            val plot = new Plot("Degree Distribution", false, "Degree", "Frequency", plotWidth, 3D/4, useIntRangeAxis = false, displayOptions = false)
            plot.setDistribution(degrees, freq)
            plot.setTitle("          Timestamp " + (t + 1))
            plotsContainer.add(plot)
        }
        plotsContainer.repaint()
    }
}