/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package guidyn.components

import javax.swing.JPanel
import javax.swing.BoxLayout
import gui.components.SpinSlider
import gui.components.SliderDescriptor
import java.awt.BorderLayout
import gui.components.RoundedSwitch
import javax.swing.JTextField
import com.alee.extended.label.WebHotkeyLabel
import com.alee.laf.spinner.WebSpinner
import com.alee.laf.combobox.WebComboBox
import javax.swing.SpinnerNumberModel
import javax.swing.DefaultComboBoxModel
import utils.SwingEvents._
import java.io.File
import algo.GeneratorParameters
import algo.MicroProba
import guidyn.components.CollapsablePanel

object Parameters extends JPanel(new VerticalLayout(5, VerticalLayout.BOTH, VerticalLayout.TOP)) {
//    /**
//     * Load graph panel
//     */
//    val loadPanel = new CollapsablePanel("Load Graph")
//    val isLoadGraphPanel = new JPanel(new BorderLayout())
//    isLoadGraphPanel.add(new WebHotkeyLabel("Load graph"), BorderLayout.WEST)
//    val isLoadGraph = new RoundedSwitch()
//    isLoadGraph.setSelected(false)
//    isLoadGraphPanel.add(isLoadGraph, BorderLayout.EAST)
//    val datasetLoadPanel = new JPanel(new BorderLayout())
//    val timestampSpinnerModel = new SpinnerNumberModel(1, 1, 1, 1)
//    val timestampSpinner = new WebSpinner(timestampSpinnerModel)
//    timestampSpinner.setVisible(false)
//    
//    val comboBoxModel = new DefaultComboBoxModel(new java.util.Vector[String]())
//    val datasetListComboBox = new WebComboBox(comboBoxModel)
//    datasetLoadPanel.add(datasetListComboBox, BorderLayout.CENTER)
//    datasetLoadPanel.add(timestampSpinner, BorderLayout.EAST)
//    datasetLoadPanel.setVisible(false)
//    
//    loadPanel.addComponent(isLoadGraphPanel)
//    loadPanel.addComponent(datasetLoadPanel)
//    
//    isLoadGraph.onAction { e =>
//        val isSelected = isLoadGraph.isSelected()
//        datasetLoadPanel.setVisible(isSelected)
//        timestampSpinner.setVisible(isSelected)
//        kField.setEditable(!isSelected)
//        nbVerticesField.setEditable(!isSelected)
//        for(a <- attributesParams) { a.setEditable(!isSelected) } 
//        nbAttributesField.setEditable(!isSelected)
//    	nbMaxInternalEdgesField.setEditable(!isSelected)
//    	nbMaxExternalEdgesField.setEditable(!isSelected)
//    	nbRepresententsField.setEditable(!isSelected)
//    	thetaField.setEditable(!isSelected)
//    	minimalTotalEdgesField.setEditable(!isSelected)
//    	closingTriangleField.setEditable(!isSelected)
//    	averageDegree.setEnabled(!isSelected)
//    	
//    	if(isSelected) {
//    	    comboBoxModel.removeAllElements()
//    	    for(n <- new File("out/").listFiles().filter(_.isDirectory()).map(a => (a.getName(), a.lastModified())).sortBy(_._2).reverse.map(_._1)) { 
//    	        comboBoxModel.addElement(n)
//	        }
//    	}
//    	
//    }
//    
//    datasetListComboBox.onAction { e =>
//        if(datasetListComboBox.getItemCount() > 0) {
//            val dataset = datasetListComboBox.getSelectedItem().toString
//	        val parameters = GeneratorParameters.read(s"out/${dataset}/parameters")
//	        loadStaticFromParamters(parameters)
//        }
//    }

//    def getParameters() = {
//        
//    }
    
    def setParametersEditable(isSelected : Boolean) = {
        kField.setEditable(!isSelected)
        nbVerticesField.setEditable(!isSelected)
        for(a <- attributesParams) { a.setEditable(!isSelected) } 
        nbAttributesField.setEditable(!isSelected)
    	nbMaxInternalEdgesField.setEditable(!isSelected)
    	nbMaxExternalEdgesField.setEditable(!isSelected)
    	nbRepresententsField.setEditable(!isSelected)
    	thetaField.setEditable(!isSelected)
    	minimalTotalEdgesField.setEditable(!isSelected)
    	//closingTriangleField.setEditable(!isSelected)
    	averageDegree.setEnabled(!isSelected)
    }
    
    def loadStaticFromParamters(parameters : GeneratorParameters) = {
        kField.setValue(parameters.k)
        nbVerticesField.setValue(parameters.nbVertices)
        for((a, i) <- attributesParams.zipWithIndex) {
            if(i < parameters.devs.size) {
                a.setValue(parameters.devs(i))
            } else {
                a.setValue(1D)
            }
    	} 
        nbAttributesField.setValue(parameters.nbAttributes)
    	nbMaxInternalEdgesField.setValue(parameters.maxEdgesWithin)
    	nbMaxExternalEdgesField.setValue(parameters.maxEdgesBetween)
    	nbRepresententsField.setValue(parameters.nbRep)
    	thetaField.setValue(parameters.theta)
    	minimalTotalEdgesField.setValue(parameters.mte)
    	//closingTriangleField.setValue(0)
    	averageDegree.setSelected(parameters.averageDegree)
    	RunPanel.seedField.setText(parameters.seed.toString)
    }
    
//    add(loadPanel)
    
    // Communities parameters
	val kField = new SpinSlider("K:", 1, 1000, 3, help = "Number of communities")
    val nbVerticesSliderDescriptor = SliderDescriptor(1, 7, 1, 1, {e : Int => Math.pow(10, e)})
    val nbVerticesField = new SpinSlider("n:", 1, 1000000, 100, help = "Number of vertices")
    
    val maxNbAttr = 100
    val attributesParams = new Array[SpinSlider](maxNbAttr)
    for(i <- 0 until maxNbAttr) {
        attributesParams(i) = new SpinSlider(s"Dev. ${i + 1}:", 0, 1000, 7, 0.01D, help = s"Standard deviation value of the attribute ${i + 1}")
        attributesParams(i).setVisible(false)
    }
    val nbAttributesField = new SpinSlider("Nb. Attr.:", 1, maxNbAttr, 2, help = "Number of attributes")
    nbAttributesField.onChange { e => for((a, i) <- attributesParams.zipWithIndex) { a.setVisible(i < nbAttributesField.value) } }
    val nbMaxInternalEdgesField = new SpinSlider("Edges Within:", 0, 1000, 3, help = "Number of edges added connecting vertices in the same community")
    val nbMaxExternalEdgesField = new SpinSlider("Edges Between:", 0, 1000, 2, help = "Number of edges added connecting vertices in different communities")
    val nbRepresententsField = new SpinSlider("Nb. Rep.:", 1, 100000, 10, help = "Number of representatives per community")
    val thetaField = new SpinSlider("Theta:", 0, 1, 0.1, 0.1D, help = "Probability for a vertex to be assigned to a random community")
    val minimalTotalEdgesField = new SpinSlider("MTE:", 0, 100000, 500, help = "Minimum number of edges in the graph")
    //val closingTriangleField = new SpinSlider("Close triangle:", 0, 100, 50, help = "Probability to close a triangle when adding an edge")
    
    val pMicroField = new SpinSlider("Proba Micro:", 0, 1, 0.1, 0.1D, help = "Probability to perform a micro update")
    val pAddVertexField = new SpinSlider("Add Vertex:", 0, 1, 0.1, 0.1D, help = "Ratio of vertices added")
    val pRemoveVertexField = new SpinSlider("Remove Vertex:", 0, 1, 0.1, 0.1D, help = "Ratio of vertices removed")
    val pUpdateAttributeField = new SpinSlider("Update Attr.:", 0, 1, 0.1, 0.1D, help = "Ratio of vertices having their attributes updated")
    val pAddBtwEdgeField = new SpinSlider("Add Btw Edge:", 0, 1, 0.1, 0.1D, help = "Ratio of between community edge added")
    val pRemoveBtwEdgeField = new SpinSlider("Remove Btw Edge:", 0, 1, 0.1, 0.1D, help = "Ratio of between community edge removed")
    val pAddWthEdgeField = new SpinSlider("Add Wth Edge:", 0, 1, 0.1, 0.1D, help = "Ratio of within community edge added")
    val pRemoveWthEdgeField = new SpinSlider("Remove Wth Edge:", 0, 1, 0.1, 0.1D, help = "Ratio of within community edge removed")
    
    val nbTimestampsField = new SpinSlider("Timestamps:", 1, 1000, 10, help = "Number of timestamps")
//    val pMacroField = new SpinSlider("Proba Macro:", 0, 1, 0.1, 0.1D, help = "Probability to perform a macro update")
    val pMergeField = new SpinSlider("Proba Merge:", 0, 1, 0.1, 0.1D, help = "Probability to merge two communities")
    val pSplitField = new SpinSlider("Proba Split:", 0, 1, 0.1, 0.1D, help = "Probability to split a community into two distinct communities")
    val pChangeField = new SpinSlider("Proba Migrate:", 0, 1, 0.1, 0.1D, help = "Probability to migrate a set of vertices")
    
    val averageDegree = new RoundedSwitch()
    val averageDegreePanel = new JPanel(new BorderLayout())
    averageDegreePanel.add(new WebHotkeyLabel("Average degree: "), BorderLayout.WEST)
    averageDegreePanel.add(averageDegree, BorderLayout.EAST)
    
    
    val communitiesPanel = new CollapsablePanel("Communities")
    communitiesPanel.addComponent(kField)
    communitiesPanel.addComponent(nbVerticesField)
    communitiesPanel.addComponent(nbRepresententsField)
    communitiesPanel.addComponent(thetaField)
    add(communitiesPanel)

    val attributesPanel = new CollapsablePanel("Attributes")
    attributesPanel.addComponent(nbAttributesField)
    for(i <- 0 until maxNbAttr) { attributesPanel.addComponent(attributesParams(i)) }
    attributesParams(0).setVisible(true)
    attributesParams(1).setVisible(true)
    add(attributesPanel)
    
    val edgesPanel = new CollapsablePanel("Edges")
    edgesPanel.addComponent(averageDegreePanel)
    edgesPanel.addComponent(nbMaxInternalEdgesField)
    edgesPanel.addComponent(nbMaxExternalEdgesField)
    //edgesPanel.addComponent(closingTriangleField)
    edgesPanel.addComponent(minimalTotalEdgesField)
    add(edgesPanel)
    
    val microDynamicPanel = new CollapsablePanel("Micro Dynamic")
    microDynamicPanel.addComponent(pMicroField)
    microDynamicPanel.addComponent(pAddVertexField)
    microDynamicPanel.addComponent(pRemoveVertexField)
    microDynamicPanel.addComponent(pUpdateAttributeField)
    microDynamicPanel.addComponent(pAddBtwEdgeField)
    microDynamicPanel.addComponent(pRemoveBtwEdgeField)
    microDynamicPanel.addComponent(pAddWthEdgeField)
    microDynamicPanel.addComponent(pRemoveWthEdgeField)
    add(microDynamicPanel)
    
    val macroDynamicPanel = new CollapsablePanel("Macro Dynamic")
    macroDynamicPanel.addComponent(nbTimestampsField)
    macroDynamicPanel.addComponent(pMergeField)
    macroDynamicPanel.addComponent(pSplitField)
    macroDynamicPanel.addComponent(pChangeField)
    add(macroDynamicPanel)
    
    // Ensure valid values for the parameters 
    nbRepresententsField.onChange { e =>
        nbRepresententsField.setValue(math.min(nbRepresententsField.value, math.ceil(nbVerticesField.value.toDouble / kField.value)))
    }
    kField.onChange { e => 
        nbRepresententsField.setValue(math.min(nbRepresententsField.value, math.ceil(nbVerticesField.value.toDouble / kField.value)))
        kField.setValue(math.min(kField.value, nbVerticesField.value))
    }
    nbVerticesField.onChange { e => 
        nbRepresententsField.setValue(math.min(nbRepresententsField.value, math.ceil(nbVerticesField.value.toDouble / kField.value)))
        kField.setValue(math.min(kField.value, nbVerticesField.value))
    }
    
    def getParameters() : GeneratorParameters = {
    	val microProba = MicroProba(
                probaMicro = Parameters.pMicroField.value,
                addVertices = Parameters.pAddVertexField.value,
                removeVertices = Parameters.pRemoveVertexField.value,
                updateAttributes = Parameters.pUpdateAttributeField.value,
                addExternalEdges = Parameters.pAddBtwEdgeField.value,
                removeExternalEdges = Parameters.pRemoveBtwEdgeField.value,
                addInternalEdges = Parameters.pAddWthEdgeField.value,
                removeInternalEdges = Parameters.pRemoveWthEdgeField.value,
                probaMerge = Parameters.pMergeField.value,
                probaSplit = Parameters.pSplitField.value,
                probaChange = Parameters.pChangeField.value)
                
        GeneratorParameters(
                seed = RunPanel.seedField.getText().toLong,
        		k = kField.value.toInt,
        		nbVertices = nbVerticesField.value.toInt,
        		nbRep = nbRepresententsField.value.toInt,
        		theta = thetaField.value,
        		nbAttributes = nbAttributesField.value.toInt,
        		devs = attributesParams.take(nbAttributesField.value.toInt).map(_.value),
        		averageDegree = averageDegree.isEnabled(),
        		maxEdgesWithin = nbMaxInternalEdgesField.value.toInt,
        		maxEdgesBetween = nbMaxExternalEdgesField.value.toInt,
        		mte = minimalTotalEdgesField.value.toInt,
        		nbTimestamps = nbTimestampsField.value.toInt,
        		dynamicParameters = Option(microProba))
    }
}