/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package guidyn.components

import javax.swing.JPanel
import java.awt.BorderLayout
import org.jfree.data.category.DefaultCategoryDataset
import org.jfree.data.xy.DefaultXYDataset
import org.jfree.chart.ChartFactory
import org.jfree.chart.ChartPanel
import javax.swing.JFrame
import java.awt.Color
import org.jfree.chart.plot.XYPlot
import org.jfree.chart.axis.LogarithmicAxis
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer
import java.awt.BasicStroke
import org.jfree.util.ShapeUtilities
import java.awt.geom.Ellipse2D
import org.jfree.chart.axis.NumberAxis
import utils.SwingEvents._
import scala.swing.event.MouseEntered
import java.awt.event.MouseEvent
import java.awt.geom.Rectangle2D
import java.awt.Rectangle
import org.w3c.dom.Document
import java.io.OutputStream
import org.jfree.data.function.LineFunction2D
import org.jfree.data.function.PowerFunction2D
import org.jfree.data.general.DatasetUtilities
import algo.Generator1
import measures.GraphMeasures
import java.awt.Dimension
import java.awt.FlowLayout
import gui.components.RoundedSwitch
import javax.swing.JLabel
import gui.components.SavableManager
import gui.components.Savable
import javax.swing.BorderFactory
import org.jfree.data.xy.XYDataset
import org.jfree.chart.labels.XYToolTipGenerator
import java.awt.Shape
import org.jfree.chart.LegendItemCollection
import org.jfree.chart.LegendItem
import org.jfree.chart.annotations.XYLineAnnotation
import scala.collection.JavaConversions._
import org.jfree.chart.title.TextTitle
import java.awt.Font

class Plot(label: String, isLogarithmic: Boolean, xLabel: String, yLabel: String, width: Int, ratio: Double, drawLine: Boolean = true, 
        useIntDomainAxis : Boolean = true, useIntRangeAxis : Boolean = false,
        legend : List[(String, Color)] = List(), drawDiagonalLine : Boolean = false, displayOptions : Boolean = true) extends JPanel(new BorderLayout()) with Savable {
    val dataset = new DefaultXYDataset()
    //    setDistribution(distribution, gen)
    val chart = ChartFactory.createXYLineChart("", "", "", dataset)
    chart.setBackgroundPaint(Color.white)
    val plot = chart.getXYPlot()
    plot.setRangePannable(false)

    if(legend.isEmpty) {
        chart.removeLegend()
    } else {
        val chartLegend = new LegendItemCollection()
	    val shape = new Rectangle(10, 10)
        for((label, color) <- legend) { chartLegend.add(new LegendItem(label, null, null, null, shape, color)) }
	    plot.setFixedLegendItems(chartLegend)
    }
    
    // Serie renderer
    val renderer = new XYLineAndShapeRenderer()
    // renderer.setPaint(Color.darkGray)
    renderer.setSeriesShape(0, new Ellipse2D.Double(-3D, -3D, 6, 6))
    renderer.setSeriesStroke(0, new BasicStroke(2))
    renderer.setSeriesPaint(0, Color.darkGray)
    renderer.setSeriesLinesVisible(0, drawLine)
    renderer.setSeriesPaint(2, Color.ORANGE)
    renderer.setToolTipGenerator(new XYToolTipGenerator() {
        def generateToolTip(arg0: XYDataset, x: Int, y: Int) = "x: " + arg0.getXValue(x, y) + " ; y: " + arg0.getYValue(x, y)
    })

    val renderer2 = new XYLineAndShapeRenderer()
    renderer2.setSeriesShape(1, new Ellipse2D.Double(-3D, -3D, 6, 6))
    renderer2.setSeriesStroke(1, new BasicStroke(2))
    renderer2.setSeriesPaint(1, Color.orange)
    
    plot.setRenderer(0, renderer)
    plot.setRenderer(1, renderer2)

    // Axis rendering
    setLogarithmic(isLogarithmic)
    
    if(useIntDomainAxis) { plot.getDomainAxis().setStandardTickUnits(NumberAxis.createIntegerTickUnits()) }
    if(useIntRangeAxis) { plot.getRangeAxis().setStandardTickUnits(NumberAxis.createIntegerTickUnits()) }
    
    plot.setBackgroundPaint(Color.white)
    plot.setRangeGridlinePaint(Color.lightGray)
    plot.setDomainGridlinePaint(Color.lightGray)

    val chartPanel = new ChartPanel(chart)
    chartPanel.setBackground(Color.white)
    chartPanel.setDisplayToolTips(true)
    
    chartPanel.setInitialDelay(0)
    chartPanel.setDismissDelay(Int.MaxValue)
    
    val optionPanel = new JPanel(new FlowLayout)
    val logX = new RoundedSwitch()
    logX.onAction(e => setLogarithmicX(logX.isSelected()))
    val logY = new RoundedSwitch()
    logY.onAction(e => setLogarithmicY(logY.isSelected()))
    optionPanel.add(new JLabel("Log X"))
    optionPanel.add(logX)
    optionPanel.add(new JLabel("Log Y"))
    optionPanel.add(logY)
    optionPanel.add(new JLabel("Update"))
    val plotAutoUpdate = new RoundedSwitch()
    optionPanel.add(plotAutoUpdate)
    add(chartPanel, BorderLayout.CENTER)
    add(optionPanel, BorderLayout.SOUTH)
    optionPanel.setBorder(BorderFactory.createLineBorder(Color.black))
    
    optionPanel.setVisible(displayOptions)
    
    def setSecondRangeAxis(label : String, distributionX: Array[Double], distributionY: Array[Double], serie: Int = 1) = {
        val dataset2 = new DefaultXYDataset()
        dataset2.addSeries(serie, Array(distributionX, distributionY))
        val axis2 = new NumberAxis(label)
        
        plot.setRangeAxis(1, axis2);
        plot.setDataset(1, dataset2);
        plot.mapDatasetToRangeAxis(1, 1);
        axis2.setAutoRange(true)
    }
    
    val plotHeight = (width * ratio).toInt
    chartPanel.onClick { e => if (e.getButton() == MouseEvent.BUTTON2) { chartPanel.restoreAutoBounds() } }
    chartPanel.setPreferredSize(new Dimension(width, plotHeight))
    chartPanel.setMaximumDrawHeight(plotHeight)
    chartPanel.setMinimumDrawHeight(plotHeight)
    chartPanel.setMaximumDrawWidth(width)
    chartPanel.setMinimumDrawWidth(width)
    chartPanel.setMouseZoomable(false)
    chartPanel.setDisplayToolTips(true)
    chartPanel.setRangeZoomable(false)
    chartPanel.setZoomInFactor(0D)
    chartPanel.setZoomOutFactor(0D)
    chartPanel.setMouseZoomable(false, false)

    def setDistribution(distributionX: Array[Double], distributionY: Array[Double], serie: Int = 1): Unit = {
        dataset.addSeries(serie, Array(distributionX, distributionY))
        if(drawDiagonalLine) {
            for(a <- plot.getAnnotations()) { plot.removeAnnotation(a.asInstanceOf[XYLineAnnotation]) }
	        plot.addAnnotation(new XYLineAnnotation(1, 1, distributionX.max, distributionY.max))
	    }
    }

    def setLogarithmic(isLogarithmic: Boolean) = {
        if (isLogarithmic) {
            plot.setDomainAxis(new LogarithmicAxis(xLabel))
            plot.setRangeAxis(new LogarithmicAxis(yLabel))
        } else {
            plot.setDomainAxis(new NumberAxis(xLabel))
            plot.setRangeAxis(new NumberAxis(yLabel))
            if(useIntDomainAxis) { plot.getDomainAxis().setStandardTickUnits(NumberAxis.createIntegerTickUnits()) }
            if(useIntRangeAxis) { plot.getRangeAxis().setStandardTickUnits(NumberAxis.createIntegerTickUnits()) }
        }
        repaint()
    }

    def setLogarithmicX(isLogarithmic: Boolean) = {
        if (isLogarithmic) { plot.setDomainAxis(new LogarithmicAxis(xLabel)) } else { plot.setDomainAxis(new NumberAxis(xLabel)) }
        if(useIntDomainAxis) { plot.getDomainAxis().setStandardTickUnits(NumberAxis.createIntegerTickUnits()) }
        repaint()
    }

    def setLogarithmicY(isLogarithmic: Boolean) = {
        if (isLogarithmic) { plot.setRangeAxis(new LogarithmicAxis(yLabel)) } else { plot.setRangeAxis(new NumberAxis(yLabel)) }
        if(useIntRangeAxis) { plot.getRangeAxis().setStandardTickUnits(NumberAxis.createIntegerTickUnits()) }
        repaint()
    }
    
    def setTitle(title : String) = {
        chartPanel.getChart().setTitle(new TextTitle(title, TextTitle.DEFAULT_FONT.deriveFont(Font.PLAIN, 12f)))
    }
    
    def outputPdf(fileName: String) = {
        utils.PdfOutput.createPdf(this, fileName, Option(getSize()))
    }

    def save() = {
        SavableManager.save(label + ".logX", logX.isSelected())
        SavableManager.save(label + ".logY", logY.isSelected())
        SavableManager.save(label + ".autoUpdate", plotAutoUpdate.isSelected())
    }
    def load() = {
        logX.setSelected(SavableManager.load(label + ".logX", isLogarithmic))
        logY.setSelected(SavableManager.load(label + ".logY", isLogarithmic))
        plotAutoUpdate.setSelected(SavableManager.load(label + ".autoUpdate", true))
    }
}
