/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package guidyn.components

import java.awt.FlowLayout
import javax.swing.JPanel
import utils.SwingEvents._
import javax.swing.JButton
import javax.swing.JTextField
import utils.Icon
import com.alee.laf.button.WebButton
import com.alee.managers.popup.WebButtonPopup
import com.alee.managers.popup.PopupWay
import algo.GeneratorParameters
import com.alee.laf.checkbox.WebCheckBox
import com.alee.laf.label.WebLabel
import guidyn.components.graph.GraphView
import utils.PdfOutput
import guidyn.Main

class SavePanel extends JPanel(new FlowLayout(FlowLayout.LEFT)) {
    val saveButton = new JButton("Save", Icon.get("disk"))
    
	val saveOptionsButton = new WebButton("Save", Icon.get("disk"))
    val popup = new WebButtonPopup(saveOptionsButton, PopupWay.downLeft)
    
    val saveNamePanel = new JPanel(new FlowLayout(FlowLayout.LEFT))
	val saveName = new JTextField(20)
    val generateSaveName = new WebButton(Icon.get("arrow_rotate_clockwise"))
    saveNamePanel.add(new WebLabel("Name:"))
    saveNamePanel.add(saveName)
    saveNamePanel.add(generateSaveName)
    
    val saveImage = new WebCheckBox("Save current graph layout")
    val saveMeasures = new WebCheckBox("Save the measures")
    val saveCommunityEvolution = new WebCheckBox("Save community dynamic")
    saveImage.setMargin(0, 5, 0, 0)
    saveMeasures.setMargin(0, 5, 0, 0)
    saveCommunityEvolution.setMargin(0, 5, 0, 0)
    
	val saveAction = new JPanel()
    saveAction.add(saveButton)
	
    
    val saveOptionsPanel = new JPanel(new VerticalLayout(5, VerticalLayout.BOTH, VerticalLayout.TOP))
	saveOptionsPanel.add(saveNamePanel)
	saveOptionsPanel.add(saveImage)
	saveOptionsPanel.add(saveMeasures)
	saveOptionsPanel.add(saveCommunityEvolution)
	saveOptionsPanel.add(saveAction)
	
	popup.setContent(saveOptionsPanel)
	
    saveButton.onAction { e =>
        val path = saveName.getText()
        if(!path.isEmpty()) {
            for(gen <- RunPanel.currentGenerator) {
                RunPanel.currentGraph.get.save(path)
                if(saveImage.isSelected()) { GraphView.saveAsPdf(path) }
                if(saveCommunityEvolution.isSelected()) { 
                    PdfOutput.createPdf( Main.communityDynamics, s"out/${path}/communityEvolution.pdf", Option(Main.communityDynamics.getSize())) 
                }
                GeneratorParameters.read(gen).save("out/" + path + "/parameters")
            }
        }
        popup.hidePopup()
    }
	
    generateSaveName.onAction { e => for(gen <- RunPanel.currentGenerator) { saveName.setText(GeneratorParameters.read(gen).toString) } }
    
	add(saveOptionsButton)
}