/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package guidyn.components.graph

import guidyn.components.CollapsablePanel
import com.alee.laf.checkbox.WebCheckBox
import gui.components.SavableManager
import utils.SwingEvents._
import gui.components.Savable
import javax.swing.JRadioButton
import javax.swing.ButtonGroup

object GraphViewOption extends CollapsablePanel("Graph View") with Savable  {
	val displayLegend = new WebCheckBox("Display Legend")
    displayLegend.onAction { e => GraphView.repaint() }
	
	val displayAllEdges = new JRadioButton("Display all edges")
	displayAllEdges.onClick { e => GraphView.repaint() }
	val displayWithinEdges = new JRadioButton("Display within edges")
	displayWithinEdges.onClick { e => GraphView.repaint() }
	val displayBetweenEdges = new JRadioButton("Display between edges")
	displayBetweenEdges.onClick { e => GraphView.repaint() }
	val displayNoEdges = new JRadioButton("Hide edges")
	displayNoEdges.onClick { e => GraphView.repaint() }
	val group = new ButtonGroup()
	group.add(displayAllEdges)
	group.add(displayWithinEdges)
	group.add(displayBetweenEdges)
	group.add(displayNoEdges)
	
    addComponent(displayAllEdges)
    addComponent(displayWithinEdges)
    addComponent(displayBetweenEdges)
    addComponent(displayNoEdges)
    addComponent(displayLegend)
	
    override def save() = {
        super.save()
        SavableManager.save("displayLegend", displayLegend.isSelected()) 
        SavableManager.save("displayAllEdges", displayAllEdges.isSelected())
        SavableManager.save("displayWithinEdges", displayWithinEdges.isSelected())
        SavableManager.save("displayBetweenEdges", displayBetweenEdges.isSelected())
        SavableManager.save("displayNoEdges", displayNoEdges.isSelected())
    }
    override def load() = {
        super.load()
        displayLegend.setSelected(SavableManager.load("displayLegend", true))
        displayAllEdges.setSelected(SavableManager.load("displayAllEdges", true))
        displayWithinEdges.setSelected(SavableManager.load("displayWithinEdges", false))
        displayBetweenEdges.setSelected(SavableManager.load("displayBetweenEdges", false))
        displayNoEdges.setSelected(SavableManager.load("displayNoEdges", false))
    }
}