/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package guidyn.components.graph

import java.awt.FlowLayout
import com.alee.extended.progress.WebStepProgress
import algo.structures.DynamicGraph
import javax.swing.JPanel
import javax.swing.JLabel
import com.alee.laf.button.WebButton
import guidyn.Main
import java.awt.Dimension
import utils.Icon
import utils.SwingEvents._
import guidyn.components.listeners.GraphUpdateListener
import java.awt.Color
import guidyn.components.listeners.TimestampUpdateListener
import javax.swing.JButton
import javax.swing.SwingWorker

object TimestampSelection extends JPanel(new FlowLayout(FlowLayout.RIGHT)) with GraphUpdateListener  {
    var curDynamicGraph : Option[DynamicGraph] = None
    def graphUpdated(g: DynamicGraph) = {
        curDynamicGraph = Option(g)
        timestamp.setStepsAmount(g.nbTimestamps)
        timestamp.setSelectedStep(0)
        timestampLabel.setText(s"Timestamp 1 ")
    }
    
    val timestamp = new WebStepProgress(0)
    timestamp.setMinimumSize(new Dimension(800,600))
    
    val timestampNext = new WebButton(Icon.get("control_play"))
    timestampNext.setUndecorated(true)
    timestampNext.onClick { e => if(timestamp.getSelectedStep() != timestamp.getStepsAmount()) { timestamp.setSelectedStep(timestamp.getSelectedStep() + 1); updateTimestamp } }
    val timestampPrev = new WebButton(Icon.get("control_back"))
    timestampPrev.setUndecorated(true)
    timestampPrev.onClick { e => if(timestamp.getSelectedStep() != 0) { timestamp.setSelectedStep(timestamp.getSelectedStep() - 1); updateTimestamp } }
    
    def updateTimestamp = { 
        val curTimestamp = timestamp.getSelectedStep()
        GraphView.curTimestamp = curTimestamp 
        timestampLabel.setText(s"Timestamp ${curTimestamp + 1} ")
        for(listener <- Main.timestampUpdateListeners) { listener.timestampUpdated(curTimestamp, curDynamicGraph.get) }
        GraphView.repaint()
    }
    timestamp.onClick(e => updateTimestamp)
    timestamp.onRelease(e => updateTimestamp)
    timestamp.onDrag(e => updateTimestamp)
    timestamp.setShowLabels(false)

    val dynamicPlay = new JButton(Icon.get("control_play_blue"))
    dynamicPlay.onAction { e =>
        val worker = new SwingWorker() {
        	override def doInBackground() : Nothing = {
				for(t <- 0 until timestamp.getStepsAmount()) {
					timestamp.setSelectedStep(t)
					updateTimestamp
					Thread.sleep(500)
				}
				1.asInstanceOf[Nothing]
	        }
			override def done() { }
        }
        worker.execute()
    }
    
    
    val timestampLabel = new JLabel("")
    
    add(dynamicPlay)
    add(timestamp)
    add(timestampLabel)
    add(timestampPrev)
    add(timestampNext)
}