/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package guidyn.components.graph.layouts

import guidyn.components.listeners.TimestampUpdateListener
import guidyn.components.listeners.GraphUpdateListener
import algo.structures.DynamicGraph
import java.awt.Dimension
import java.awt.geom.Point2D
import edu.uci.ics.jung.algorithms.layout.Layout
import guidyn.components.RunPanel
import algo.structures.Graph
import edu.uci.ics.jung.graph.UndirectedSparseGraph
import guidyn.components.graph.GraphView

class PlotLayoutManager extends TimestampUpdateListener with GraphUpdateListener {
    var curTimestamp = 0
	var curDynamicGraph : Option[DynamicGraph] = None
	var minAttrs = Array[Double]()
    var maxAttrs = Array[Double]()
	
    def timestampUpdated(timestamp : Int, g : DynamicGraph) {
        curTimestamp = timestamp
        curDynamicGraph = Option(g)
    }
    def graphUpdated(g : DynamicGraph) = {
        curTimestamp = 0
        curDynamicGraph = Option(g)
        
        // Update min and max attribute values
        val nbAttributes = g.nbAttributes
        minAttrs = Array.fill(nbAttributes)(Double.MaxValue)
        maxAttrs = Array.fill(nbAttributes)(Double.MinValue)
        for(graph <- g.graphs; v <- graph.vertices; i <- 0 until nbAttributes) {
            minAttrs(i) = math.min(minAttrs(i), graph.attr(v)(i))
            maxAttrs(i) = math.max(maxAttrs(i), graph.attr(v)(i))
        }
        
        // Generate the layouts
        layouts = (0 until g.nbTimestamps).toArray.map { i =>
             new PlotLayout(g.graphs(i), this)
        }
    }
    
    var layouts = Array[PlotLayout]()
    def getLayoutForTimestamp() = layouts(curTimestamp)
    def getLayoutForTimestamp(timestamp : Int) = layouts(timestamp)
}

class PlotLayout(var graph : Graph, layoutManager : PlotLayoutManager) extends Layout[Int,Int] {
    var border : Double = 30
    var isFirst = true
    val dimension = new Dimension(0, 0)
    val minAttrs = new Array[Double](2)
    val maxAttrs = new Array[Double](2)
    
    def getSize() = if(isFirst) { isFirst = false; null } else { dimension }
    def initialize() = {}
    def isLocked(v : Int) = true
    def lock(v : Int, state: Boolean) = {}
    def reset() = {}
    override def getGraph() = { GraphView.jungGraph }
    def setGraph(g : edu.uci.ics.jung.graph.Graph[Int,Int]) { }
    
    def getXY(v2 : Int) : Point2D = {
        val v = v2.toString.toInt
        if(graph.nbAttributes >= 2) {
	        val w = dimension.getWidth()
	        val h = dimension.getHeight()
	        val xn = (graph.attr(v)(0) - layoutManager.minAttrs(0)) / (layoutManager.maxAttrs(0) - layoutManager.minAttrs(0))
	        val yn = (graph.attr(v)(1) - layoutManager.minAttrs(1)) / (layoutManager.maxAttrs(1) - layoutManager.minAttrs(1))
	        val x = (xn * (w - (2 * border)) + border)
	        val y = (yn * (h - (2 * border)) + border)
	        new Point2D.Double(x, y)
        } else {
            new Point2D.Double(0, 0)
        }
    }
    def setInitializer(initializer: org.apache.commons.collections15.Transformer[Int,Point2D]) = { }
    def setLocation(v : Int, location : Point2D) = location.setLocation(getXY(v))
    def setSize(d : Dimension) = { dimension.setSize(d.getWidth() - border, d.getHeight() - border) }
    def transform(v2 : Int) : Point2D = { getXY(v2) }
}