/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package layout.yifanHu

import utils.Timer

class BarnesHut(force : AbstractForce, var theta : Float = 1.2f) {

    /* Calculates the ForceVector on node against every other node represented
     * in the tree with respect to force.
     */
    def calculateForceOld(node : Node, tree : QuadTree) : ForceVector = {
        if (tree.mass <= 0) {
            return null;
        }
        
        val distance = Math.hypot(node.x - tree.centerMassX, node.y - tree.centerMassY).toFloat

        if (tree.isLeaf || tree.mass == 1) {
            // this is probably the case where tree has only the node.
            if (distance < 1e-8) {
                return null;
            }
            return force.calculateForce(node, tree);
        }

        if (distance * theta > tree.size) {
            val f = force.calculateForce(node, tree, distance)
            f.multiply(tree.mass);
            return f;
        }
        
        val f = new ForceVector();
        for (child <- tree.children) {
        	f.add(calculateForce(node, child));
        }
        return f;
    }
    
    def calculateForce(node : Node, tree : QuadTree) : ForceVector = {
        var curForce : ForceVector = null
        if (tree.mass > 0) {
            Timer.startTimer("dist")
            val distance = Math.hypot(node.x - tree.centerMassX, node.y - tree.centerMassY).toFloat
            Timer.stopTimer("dist")
	        if (tree.isLeaf || tree.mass == 1) {
	            if (distance >= 1e-8) { curForce = force.calculateForce(node, tree) }
	        } else if (distance * theta > tree.size) {
	            curForce = force.calculateForce(node, tree, distance)
	            curForce.multiply(tree.mass);
	        } else {
	            curForce = new ForceVector();
		        for (child <- tree.children) { curForce.add(calculateForce(node, child)); }
	        }
        }
        curForce
    }
}