/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package layout.yifanHu

import scala.collection.mutable.ArrayBuffer

class QuadTree(var posX : Float, var posY : Float, val size : Float, var maxLevel : Int) extends Node {

//    var posX : Float = 0f
//    var posY : Float = 0f
//    var size : Float = 0f
    var centerMassX : Float = 0f
    var centerMassY : Float = 0f
    var mass : Int = 0
//    var maxLevel : Int = 0
    var add : AddBehaviour = new FirstAdd()
    var children = List[QuadTree]()
    var isLeaf = true
    
    val eps = 1e-6
    
//    public float size() {
//        return size;
//    }

    def divideTree() = {
        var childSize = size / 2;

        children = List[QuadTree]()
        children = new QuadTree(posX + childSize, posY + childSize, childSize, maxLevel - 1) :: children
        children = new QuadTree(posX, posY + childSize, childSize, maxLevel - 1) :: children
        children = new QuadTree(posX, posY, childSize, maxLevel - 1) :: children
        children = new QuadTree(posX + childSize, posY, childSize, maxLevel - 1) :: children

        isLeaf = false;
    }

    def addToChildren(node : Node) : Boolean = {
        for (q <- children) {
            if (q.addNode(node)) {
                return true;
            }
        }
        return false;
    }

    def assimilateNode(node : Node) {
        centerMassX = (mass * centerMassX + node.x) / (mass + 1);
        centerMassY = (mass * centerMassY + node.y) / (mass + 1);
        mass += 1
    }

    def addNode(node : Node) : Boolean = {
        if (posX <= node.x && node.x <= posX + size && posY <= node.y && node.y <= posY + size) {
            add.addNode(node)
        } else {
            return false;
        }
    }
    
    /*

    /**
     * @return the isLeaf
     */
    public boolean isIsLeaf() {
        return isLeaf;
    }
    */

    class FirstAdd extends AddBehaviour {
    	def addNode(node : Node) : Boolean = {
            mass = 1;
            centerMassX = node.x
            centerMassY = node.y

            if (maxLevel == 0) {
                add = new LeafAdd();
            } else {
                add = new SecondAdd();
            }
            true
        }
    }

    class SecondAdd extends AddBehaviour {
        def addNode(node : Node) : Boolean = {
            divideTree()
            add = new RootAdd()
            addToChildren(QuadTree.this);
            add.addNode(node);
        }
    }

    class LeafAdd extends AddBehaviour {
        def addNode(node : Node) : Boolean = {
            assimilateNode(node)
            true;
        }
    }

    class RootAdd extends AddBehaviour {
        def addNode(node : Node) : Boolean = {
            assimilateNode(node)
            addToChildren(node)
        }
    }
}

object QuadTreeBuilder {
    def buildTree(graph : Graph, maxLevel : Int) : QuadTree = {
        var minX = Float.MaxValue
        var maxX = Float.MinValue
        var minY = Float.MaxValue
        var maxY = Float.MinValue

        for (node <- graph.nodes) {
            minX = Math.min(minX, node.x)
            maxX = Math.max(maxX, node.x)
            minY = Math.min(minY, node.y)
            maxY = Math.max(maxY, node.y)
        }

        val size = Math.max(maxY - minY, maxX - minX)
        val tree = new QuadTree(minX, minY, size, maxLevel)
        for (node <- graph.nodes) {
            tree.addNode(node)
        }

        return tree;
    }
}

trait AddBehaviour {
    def addNode(node : Node) : Boolean
}