/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/


package scalargs.argtypes
import collection.mutable.ListBuffer

/**
 * Created by IntelliJ IDEA.
 * User: nico
 * Date: 6 juin 2010
 * Time: 13:42:20
 * To change this template use File | Settings | File Templates.
 */
class ArgType(
        val name : String,
        var names : List[String] = List(),
        var help : String = "",
        var choices : List[String] = List(),
        var nbRequiredParams : Int = 1,
        var nbOptionalParams : Int = 0,
        var metaVar : String = "ARG",
        var group : String = "",
        var showDefault : Boolean = true,
        var showParams : Boolean = true) {

    var params = new ListBuffer[String]()
    var isSet = false
    // var parser : (Arg, AnyRef) => Boolean = _
    var errorMessage = ""
    var isRequired : Boolean = !name.startsWith("-")
    var isPositional : Boolean = !name.startsWith("-")
    var allNames = name :: names

    // Positional arguments do not have parameters
    if (isPositional) {
        nbRequiredParams = 1
        nbOptionalParams = 0
    }

    if (group == "") {
        if (isPositional) {
            group = "positional arguments"
        } else {
            group = "optional arguments"
        }
    }


    def getArgParams() : String = {
        if (!showParams || (nbOptionalParams == 0 && nbRequiredParams == 0)) {
            return ""
        }
        var argsParam = ""
        if (nbRequiredParams == 1) {
            argsParam = metaVar
        } else {
            argsParam = 1.to(nbRequiredParams).map(metaVar + _).mkString(",")
        }
        if (nbOptionalParams > 0) {
            if (nbRequiredParams == 0) {
                argsParam += "["
            } else {
                argsParam += "[,"
            }
        }

        if (nbOptionalParams == Integer.MAX_VALUE) {
            argsParam += metaVar + "]*"
        } else {
            if (nbOptionalParams == 1 && nbRequiredParams == 0) {
                argsParam = "[" + metaVar
            } else {
                argsParam += (nbRequiredParams + 1).to(nbOptionalParams + nbRequiredParams).map(metaVar + _).mkString("[,")
            }
            argsParam += ("]" * nbOptionalParams)
        }
        return " " + argsParam
    }

    def getArgUsage() : String = {
        var argUsage = if(isRequired) "" else "["
        argUsage += name + getArgParams()
        argUsage += (if(isRequired) " " else "] ")
        return argUsage
    }

    def addParam(param : String) : Unit = {
        params += param
    }

    def hasValidNumberOfParams() : Boolean = {
        if (params.size >= nbRequiredParams) {
            return nbOptionalParams == Integer.MAX_VALUE || params.size <= nbRequiredParams + nbOptionalParams
        }
        return false
    }

    def canAddAnotherParam() : Boolean = {
        return nbOptionalParams == Integer.MAX_VALUE || params.size < nbRequiredParams + nbOptionalParams
    }
}
