/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package utils

import java.awt.Color

/**
 * This class is used to generate colors that "match well"
 */
object Colors {
    
    val indexColors128 = Array[String](
        "#000000", "#FFFF00", "#1CE6FF", "#FF34FF", /* "#FF4A46", */ "#008941", "#006FA6", "#A30059",
        "#FFDBE5", "#7A4900", "#0000A6", "#63FFAC", "#B79762", "#004D43", "#8FB0FF", "#997D87",
        "#5A0007", "#809693", "#FEFFE6", "#1B4400", "#4FC601", "#3B5DFF", "#4A3B53", "#FF2F80",
        "#61615A", "#BA0900", "#6B7900", "#00C2A0", "#FFAA92", "#FF90C9", "#B903AA", "#D16100",
        "#DDEFFF", "#000035", "#7B4F4B", "#A1C299", "#300018", "#0AA6D8", "#013349", "#00846F",
        "#372101", "#FFB500", "#C2FFED", "#A079BF", "#CC0744", "#C0B9B2", "#C2FF99", "#001E09",
        "#00489C", "#6F0062", "#0CBD66", "#EEC3FF", "#456D75", "#B77B68", "#7A87A1", "#788D66",
        "#885578", "#FAD09F", "#FF8A9A", "#D157A0", "#BEC459", "#456648", "#0086ED", "#886F4C",

        "#34362D", "#B4A8BD", "#00A6AA", "#452C2C", "#636375", "#A3C8C9", "#FF913F", "#938A81",
        "#575329", "#00FECF", "#B05B6F", "#8CD0FF", "#3B9700", "#04F757", "#C8A1A1", "#1E6E00",
        "#7900D7", "#A77500", "#6367A9", "#A05837", "#6B002C", "#772600", "#D790FF", "#9B9700",
        "#549E79", "#FFF69F", "#201625", "#72418F", "#BC23FF", "#99ADC0", "#3A2465", "#922329",
        "#5B4534", "#FDE8DC", "#404E55", "#0089A3", "#CB7E98", "#A4E804", "#324E72", "#6A3A4C",
        "#83AB58", "#001C1E", "#D1F7CE", "#004B28", "#C8D0F6", "#A3A489", "#806C66", "#222800",
        "#BF5650", "#E83000", "#66796D", "#DA007C", "#FF1A59", "#8ADBB4", "#1E0200", "#5B4E51",
        "#C895C5", "#320033", "#FF6832", "#66E1D3", "#CFCDAC", "#D0AC94", "#7ED379", "#012C58")
    val colors128 = indexColors128.map(c => Color.decode(c))
    
    val indexColors27 = Array[String](
        "#fce94f", "#fcaf3e", "#e9b96e", "#8ae234", "#729fcf", "#ad7fa8", "#ef2929", "#eeeeec", "#888a85",
        "#edd400", "#f57900", "#c17d11", "#73d216", "#3465a4", "#75507b", "#cc0000", "#d3d7cf", "#555753",
        "#c4a000", "#ce5c00", "#8f5902", "#4e9a06", "#204a87", "#5c3566", "#a40000", "#babdb6", "#2e3436")
    val colors27 = indexColors27.map(c => Color.decode(c))
    
    val indexColors24 = Array[String](
        "#fce94f", "#e9b96e", "#8ae234", "#729fcf", "#ad7fa8", "#ef2929", "#eeeeec", "#888a85",
        "#edd400", "#c17d11", "#73d216", "#3465a4", "#75507b", "#cc0000", "#d3d7cf", "#555753",
        "#c4a000", "#8f5902", "#4e9a06", "#204a87", "#5c3566", "#a40000", "#babdb6", "#2e3436")
    val colors24 = indexColors24.map(c => Color.decode(c))
    
        
    def getMatchingColors(nbColors : Int, saturation : Float = 0.8f, value : Float = 0.8f) : Array[Color] = {
        if(nbColors <= colors24.size) {
            colors128.take(nbColors).reverse
        } else if(nbColors <= colors27.size) {
            colors27.take(nbColors).reverse
        } else if(nbColors <= colors128.size) {
            colors128.take(nbColors)
        } else {
            (0 until nbColors).map ({ idx =>
	        Color.getHSBColor(((idx.toFloat + 1) / (nbColors + 1)), saturation, value)
            }).toArray
        }
        
    }
    
    def getInverseColor(c : Color) : Color = {
        val hsb = Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), null)
        new Color(Color.HSBtoRGB(math.abs(hsb(0) - 1), math.abs(hsb(1) - 1), hsb(2)))
    }
    
    def getTextColor(c : Color) : Color = {
        if ((c.getRed() * 0.299 + c.getGreen() * 0.587 + c.getBlue() * 0.114) < 186) { Color.white } else { Color.black } 
    }
}