/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package utils

/**
 * Compute expected runtime of a task repeating several time
 * It is based on the average runtime of the task previously done
 */
class ExpectedRuntime(val nbIteration : Int, val nbIterationBeforeUpdate : Int = 1) {
    var expectedRuntime : Option[Double] = None
    var expectedRemainingRuntime : Option[Double] = None
    
    /**
     * This percentage is computed such that the amount cannot decrease
     * which would be strange for the user
     * It simply does not change if the computed percentage does not increase
     */
    var increasingPercentageDone : Option[Double] = None
    var percentageDone : Option[Double] = None
    
	private var deltaSum = 0L
    private var curDelta = 0L
    private var nbIterDone = 0
    
    def startTask() = {
	    curDelta = System.currentTimeMillis()
	}
    
    def endTask() = {
        deltaSum += (System.currentTimeMillis() - curDelta)
        nbIterDone += 1
        val averageTimeRequired = deltaSum.toDouble / nbIterDone
        
        if(nbIterDone >= nbIterationBeforeUpdate) {
            expectedRemainingRuntime = Option((nbIteration - nbIterDone) * averageTimeRequired)
    		expectedRuntime = Option(nbIteration * averageTimeRequired)
    		percentageDone = Option((expectedRemainingRuntime.get / expectedRuntime.get) * 100)
    		if(increasingPercentageDone.getOrElse(0D) <= percentageDone.get) {
    		    increasingPercentageDone = percentageDone
    		}
        }
    }
}