/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package utils

import org.apache.batik.dom.GenericDOMImplementation
import org.apache.batik.svggen.SVGGraphics2D
import java.io.Writer
import java.io.FileOutputStream
import java.io.IOException
import java.awt.Color
import java.awt.Dimension
import java.awt.RenderingHints
import javax.swing.JPanel
import java.io.OutputStreamWriter
import java.io.File
import org.apache.fop.svg.PDFGraphics2D
import org.apache.fop.svg.PDFTranscoder
import org.apache.batik.transcoder.TranscoderOutput
import org.apache.batik.transcoder.TranscoderInput

object PdfOutput {

    def createPdf(panel: JPanel, outPath: String, dimension : Option[Dimension] = None) = {
        val domImpl = GenericDOMImplementation.getDOMImplementation()
        val document = domImpl.createDocument("http://www.w3.org/2000/svg", "svg", null)
        val svgGenerator = new SVGGraphics2D(document)

        // computeDimension();
        val dim = dimension.getOrElse(new Dimension(800, 800))
        panel.setMinimumSize(dim)
        panel.setPreferredSize(dim)
        svgGenerator.setSVGCanvasSize(dim)
        svgGenerator.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON)
        // panel.setFont(panel.getFont().deriveFont(Font.BOLD))
        panel.setBackground(Color.WHITE)
        
        panel.paint(svgGenerator)

        // Save SVG document
        val useCSS = true
        val fos = new FileOutputStream(new File(".graph-tmp.svg"))
        val out = new OutputStreamWriter(fos, "UTF-8")
        svgGenerator.stream(out, useCSS)
        
        // Convert to pdf
        val pa = new File(".graph-tmp.svg").toURL().toString()
        val inputSvgImage = new TranscoderInput(pa)
        val pdfOstream = new FileOutputStream(outPath)
        val outputPdfFile = new TranscoderOutput(pdfOstream)
        val transcoder = new PDFTranscoder()
        transcoder.transcode(inputSvgImage, outputPdfFile)
        pdfOstream.flush()
        pdfOstream.close()
    }
}