/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package utils

import java.awt.event.ActionEvent
import java.awt.event.ActionListener
import java.awt.event.KeyAdapter
import java.awt.event.KeyEvent
import java.awt.event.MouseAdapter
import java.awt.event.MouseEvent
import java.awt.event.MouseMotionAdapter
import java.awt.event.MouseWheelEvent
import java.awt.event.MouseWheelListener

import com.alee.laf.checkbox.WebCheckBox
import com.alee.laf.combobox.WebComboBox
import com.alee.laf.slider.WebSlider

import gui.components.RoundedSwitch
import javax.swing.JButton
import javax.swing.JCheckBox
import javax.swing.JComponent
import javax.swing.event.ChangeListener
import javax.swing.event.ChangeEvent

object SwingEvents {
    implicit def convertToActionable(c: JButton) = new SActionable(c)
    implicit def convertToActionable(c: RoundedSwitch) = new SActionable(c)
    implicit def convertToActionable(c: WebComboBox) = new SActionable(c)
    implicit def convertToActionable(c: JCheckBox) = new SActionable(c)
    class SActionable(c : JComponent) {
        val actionHandler = (c.getClass().getMethods().toList ::: c.getClass().getDeclaredMethods().toList).find(e => e.getName() == "addActionListener")
        def onAction(f : ActionEvent => Unit) = {
            actionHandler.foreach { a =>
                val actionListener = new ActionListener() { def actionPerformed(e : ActionEvent) : Unit = { f(e) } }
                a.invoke(c, Array(actionListener) : _*)
            }
        }
	}
    
    
    implicit def convertToChangeable(c: WebSlider) = new SChangeable(c)
    class SChangeable(c : JComponent) {
        val actionHandler = (c.getClass().getMethods().toList ::: c.getClass().getDeclaredMethods().toList).find(e => e.getName() == "addChangeListener")
        def onChange(f : ChangeEvent => Unit) = {
            actionHandler.foreach { a =>
                val changeListener = new ChangeListener() { def stateChanged(e : ChangeEvent) : Unit = { f(e) } }
                a.invoke(c, Array(changeListener) : _*)
            }
        }
	}
    
    implicit def convertComponent(c : JComponent) = new SComponent(c)
    class SComponent(c : JComponent) {
        // Mouse click
	    def onClick(f : MouseEvent => Unit) = { c.addMouseListener(new MouseAdapter() { override def mouseClicked(e : MouseEvent) : Unit = { f(e) } }) }
	    def onRelease(f : MouseEvent => Unit) = { c.addMouseListener(new MouseAdapter() { override def mouseReleased(e : MouseEvent) : Unit = { f(e) } }) }
	    def onPress(f : MouseEvent => Unit) = { c.addMouseListener(new MouseAdapter() { override def mousePressed(e : MouseEvent) : Unit = { f(e) } }) }
	    def onExit(f : MouseEvent => Unit) = { c.addMouseListener(new MouseAdapter() { override def mouseExited(e : MouseEvent) : Unit = { f(e) } }) }
	    def onEnter(f : MouseEvent => Unit) = { c.addMouseListener(new MouseAdapter() { override def mouseEntered(e : MouseEvent) : Unit = { f(e) } }) }
	    
	    // Mouse drag
	    def onDrag(f : MouseEvent => Unit) = { c.addMouseMotionListener(new MouseMotionAdapter() { override def mouseDragged(e : MouseEvent) : Unit = { f(e) } }) }
	    def onMove(f : MouseEvent => Unit) = { c.addMouseMotionListener(new MouseMotionAdapter() { override def mouseMoved(e : MouseEvent) : Unit = { f(e) } }) }
	    
	    // Mouse wheel
	    def onMouseWheelMove(f : MouseWheelEvent => Unit) = {c.addMouseWheelListener(new MouseWheelListener() { def mouseWheelMoved(e : MouseWheelEvent) : Unit = {f(e)} } )}
	    
	    // Keys
	    def onKeyPress(f : KeyEvent => Unit) = {c.addKeyListener(new KeyAdapter() { override def keyPressed(e : KeyEvent) : Unit = {f(e)} } )}
	    def onKeyRelease(f : KeyEvent => Unit) = {c.addKeyListener(new KeyAdapter() { override def keyReleased(e : KeyEvent) : Unit = {f(e)} } )}
	    def onKeyType(f : KeyEvent => Unit) = {c.addKeyListener(new KeyAdapter() { override def keyTyped(e : KeyEvent) : Unit = {f(e)} } )}
	    
	}
}