/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package utils
import scala.collection.mutable.HashMap
import java.io.PrintStream
import java.io.FileOutputStream
import java.io.File

object Timer {
	private var deltas = new HashMap[String, Double]()
	private var timers = new HashMap[String, Double]()
	
	def startTimer(timerName : String) = {
		deltas(timerName) = System.currentTimeMillis()
	}
	
	def stopTimer(timerName : String) = {
		var curTimer = System.currentTimeMillis() - deltas(timerName)
		timers(timerName) = timers.getOrElse(timerName, 0D) + curTimer
	}
	
	def getTimer(timerName : String) = timers(timerName)
	def clearTimer(timerName : String) = {
	    timers(timerName) = 0D
	}
	
	def clearAllTimers() = {
	    timers.clear()
	}
	
	def printTimers() = {
		println("# \n# Timers")
		timers.toList.sortBy({_._2}).foreach { a =>
		  	println("# " + a._1 + ":\t" + a._2 / 1000)
		}
	}
	
	def time[R](timerName : String)(block: => R) : R = {
	    deltas(timerName) = System.currentTimeMillis()
	    val result = block
	    var curTimer = System.currentTimeMillis() - deltas(timerName)
		timers(timerName) = timers.getOrElse(timerName, 0D) + curTimer
		result
	}
}