/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package xps

import scala.io.Source
import utils.Stat
import java.io.PrintStream
import java.io.FileOutputStream
import java.io.File

object ParseRuntime {
    case class Runtimes(var base : Double, 
            var addVert : Double, var removeVert : Double, var updateAttr : Double, 
            var addWth : Double, var removeWth : Double, var addBtw : Double, var removeBtw : Double,
            var migrate : Double, var split : Double, var merge : Double, 
            var updateRepresentatives : Double, var total : Double)
    
    def main(args: Array[String]): Unit = {
        val console = System.out
        System.setOut(new PrintStream(new FileOutputStream(new File("xp/runtimeDyn/runtimeAll.dat"))))
        var i = 0
        val nbSeeds = 2
        val nbTimestamps = 20
        var data = Array.fill(nbSeeds, nbTimestamps)(Runtimes(0,0,0,0,0,0,0,0,0,0,0,0,0))
        var curTimestamp = -1
        var curSeed = -1
        var baseTime = 0D
        var totalTime = 0D
        for(line <- Source.fromFile("runtimes").getLines) {
//            println(line)
            if(line.startsWith("t")) {
                totalTime = line.split("\t")(1).toDouble
                if(line.startsWith("t1.0")) { baseTime = totalTime }
                curTimestamp += 1
                data(curSeed)(curTimestamp).base = baseTime
                data(curSeed)(curTimestamp).total = totalTime
            } else if(line.startsWith("Seed")) {
                curSeed += 1
                curTimestamp = -1
            } else {
                if(line.startsWith("#") && line.contains(":\t")) {
                    val value = line.split(":\t")(1).toDouble
                    val cur = data(curSeed)(curTimestamp)
                    if(line.startsWith("# Micro - Add vertices")) { cur.addVert = value }
                    if(line.startsWith("# Micro - Remove vertices")) { cur.removeVert = value }
                    if(line.startsWith("# Micro - Remove Between Edges")) { cur.removeBtw = value }
                    if(line.startsWith("# Micro - Remove Within Edges")) { cur.removeWth = value }
                    if(line.startsWith("# Micro - Add Within Edges")) { cur.addWth = value }
                    if(line.startsWith("# Micro - Add Between Edges")) { cur.addBtw = value }
                    if(line.startsWith("# Micro - Update attributes")) { cur.updateAttr = value }
                    if(line.startsWith("# Macro - Migrate")) { cur.migrate = value }
                    if(line.startsWith("# Macro - Split")) { cur.split = value }
                    if(line.startsWith("# Macro - Merge")) { cur.merge = value }
                    if(line.startsWith("# Update representatives")) { cur.updateRepresentatives = value }
                }
            }
        }
        
        for(t <- 0 until nbTimestamps) {
            val (addVert, addVertDev) = Stat.meanStdDev(data.map(_(t).addVert))
            val (removeVert, removeVertDev) = Stat.meanStdDev(data.map(_(t).removeVert))
            val (removeBtw, removeBtwDev) = Stat.meanStdDev(data.map(_(t).removeBtw))
            val (removeWth, removeWthDev) = Stat.meanStdDev(data.map(_(t).removeWth))
            val (addWth, addWthDev) = Stat.meanStdDev(data.map(_(t).addWth))
            val (addBtw, addBtwDev) = Stat.meanStdDev(data.map(_(t).addBtw))
            val (updateAttr, updateAttrDev) = Stat.meanStdDev(data.map(_(t).updateAttr))
            val (migrate, migrateDev) = Stat.meanStdDev(data.map(_(t).migrate))
            val (split, splitDev) = Stat.meanStdDev(data.map(_(t).split))
            val (merge, mergeDev) = Stat.meanStdDev(data.map(_(t).merge))
            val (updateRepresentatives, updateRepresentativesDev) = Stat.meanStdDev(data.map(_(t).updateRepresentatives))
            val (base, baseDev) = Stat.meanStdDev(data.map(_(t).base))
            val (total, totalDev) = Stat.meanStdDev(data.map(_(t).total))
            
            
            // This is ugly and beautiful at the same time :)
            
            println(List(t + 1, 
                    base, 
                    removeVert + base,
                    addVert + removeVert + base,
                    updateAttr + addVert + removeVert + base,
                    removeWth + updateAttr + addVert + removeVert + base, 
                    addWth + removeWth + updateAttr + addVert + removeVert + base, 
                    removeBtw + addWth + removeWth + updateAttr + addVert + removeVert + base,
                    addBtw + removeBtw + addWth + removeWth + updateAttr + addVert + removeVert + base, 
                    migrate + addBtw + removeBtw + addWth + removeWth + updateAttr + addVert + removeVert + base, 
                    merge + migrate + addBtw + removeBtw + addWth + removeWth + updateAttr + addVert + removeVert + base,
                    split + merge + migrate + addBtw + removeBtw + addWth + removeWth + updateAttr + addVert + removeVert + base, 
                    updateRepresentatives + split + merge + migrate + addBtw + removeBtw + addWth + removeWth + updateAttr + addVert + removeVert + base, 
                    total, 0).mkString("\t"))
        }
        
        val totalSum = (data.map(_(nbTimestamps - 1).total)).sum.toDouble
        
        val addVertSum = (data.map(_(nbTimestamps - 1).addVert)).sum.toDouble / totalSum
        val removeVertSum = (data.map(_(nbTimestamps - 1).removeVert)).sum.toDouble / totalSum
        val removeBtwSum = (data.map(_(nbTimestamps - 1).removeBtw)).sum.toDouble / totalSum
        val removeWthSum = (data.map(_(nbTimestamps - 1).removeWth)).sum.toDouble / totalSum
        val addWthSum = (data.map(_(nbTimestamps - 1).addWth)).sum.toDouble / totalSum
        val addBtwSum = (data.map(_(nbTimestamps - 1).addBtw)).sum.toDouble / totalSum
        val updateAttrSum = (data.map(_(nbTimestamps - 1).updateAttr)).sum.toDouble / totalSum
        val migrateSum = (data.map(_(nbTimestamps - 1).migrate)).sum.toDouble / totalSum
        val splitSum = (data.map(_(nbTimestamps - 1).split)).sum.toDouble / totalSum
        val mergeSum = (data.map(_(nbTimestamps - 1).merge)).sum.toDouble / totalSum
        val updateRepresentativesSum = (data.map(_(nbTimestamps - 1).updateRepresentatives)).sum.toDouble / totalSum
        val baseSum = (data.map(_(nbTimestamps - 1).base)).sum.toDouble / totalSum
        
        System.out.flush()
        System.setOut(console)
        println(f"${addVertSum * 100}%.1f".replaceAll(",", ".") + " %")
        println(f"${removeVertSum * 100}%.1f".replaceAll(",", ".") + " %")
        println(f"${removeBtwSum * 100}%.1f".replaceAll(",", ".") + " %")
        println(f"${removeWthSum * 100}%.1f".replaceAll(",", ".") + " %")
        println(f"${addWthSum * 100}%.1f".replaceAll(",", ".") + " %")
        println(f"${addBtwSum * 100}%.1f".replaceAll(",", ".") + " %")
        println(f"${updateAttrSum * 100}%.1f".replaceAll(",", ".") + " %")
        println(f"${migrateSum * 100}%.1f".replaceAll(",", ".") + " %")
        println(f"${splitSum * 100}%.1f".replaceAll(",", ".") + " %")
        println(f"${mergeSum * 100}%.1f".replaceAll(",", ".") + " %")
        println(f"${updateRepresentativesSum * 100}%.1f".replaceAll(",", ".") + " %")
        println(f"${baseSum * 100}%.1f".replaceAll(",", ".") + " %")
        
            
    }
}