/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package xps

import algo.MicroProba
import algo.GeneratorDyn
import measures.GraphMeasures
import utils.Timer

object TestShortesPath {

    def main(args: Array[String]): Unit = {
        val microProbaTmp = MicroProba(probaMicro = 0D, addVertices = 0.1D, removeVertices = 0.1D, updateAttributes = 0.1D, removeInternalEdges = 0.1D, 
            				addInternalEdges = 0.1D,
	                		removeExternalEdges = 0.1D, addExternalEdges = 0.1D, probaMerge = 0.0D, probaSplit = 0.0D, probaChange = 0.0D)
        val genTmp = new GeneratorDyn(k = 100, nbVertices = 5000, dev = Array(1D), theta = 0, probaCloseTriangle = 0, nbRep = 10,
			                avgWthDegree = 6, avgBtwDegree = 3, minimumTotalEdges = 0, seed = Option(1), nbTimestamps = 1, useAverageDegree = false, 
			                microProba = microProbaTmp)
        val graph = genTmp.generateDynamicGraph(computeDynamicMeasure = false)
        val m = new GraphMeasures(graph.graphs(0), Array(1))
        println(m.avgShortestPath)
        val m1 = new GraphMeasures(graph.graphs(0), Array(1))
        println(m1.avgShortestPath)
        val m2 = new GraphMeasures(graph.graphs(0), Array(1))
        Timer.startTimer("Sum shortest path")
        println(m2.avgShortestPath)
        Timer.stopTimer("Sum shortest path")
        Timer.printTimers
        
        graph.graphs(0).writeGraphCsv("graphTest.csv")
    }

}