/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package xps

import algo.Generator1
import measures.GraphMeasures
import algo.Output
import algo.structures.CommunitiesInformation
import java.io.File
import java.io.PrintWriter
import scala.collection.mutable.HashMap
import utils.Serializer
import utils.Latex
import utils.LatexHeader
import utils.LatexPackage

object XpOneDataset {
    val measures = List(
        Measure.AvgDegree,
        Measure.MaxDegree,
        Measure.AvgShortestPathLength,
        Measure.BetweenInertiaRatio,
        Measure.ClusteringCoefficient,
        Measure.Diameter,
        Measure.HomophilyBetween,
        Measure.HomophilyWithin,
        Measure.Modularity,
        Measure.NbExternalEdges,
        Measure.NbInternalEdges,
        Measure.NbEdges,
        Measure.WithinInertiaRatio,
        Measure.ExpectedHomophileEdges,
        Measure.FractionOfHomophileEdges)

    val defNbRep = 100

    val basePath = "./xp3/"
    new File(basePath).mkdirs()
    val latex = new Latex(s"${basePath}results.tex", Option(LatexHeader("Results NbRep = " + defNbRep, List(
            LatexPackage("graphics"), LatexPackage("graphicx"), LatexPackage("caption"), LatexPackage("booktabs"), LatexPackage("subfigure")))))
//    latex.printItemize(List("NbRep : 100", "degree (plus de log)", "Avec uniquement distance"))
    
    
    val refSetting = ParamSetting("Ref", k = 3, nbVertices = 1000, dev = Array(7D, 1D), theta = 0D, nbRep = defNbRep,
        maxInternalEdges = 6, maxExternalEdges = 3, minimumTotalEdges = 0, seed = Option(1))
    val refSetting2 = ParamSetting("Dev1", k = 3, nbVertices = 1000, dev = Array(7D, 7D), theta = 0D, nbRep = defNbRep,
        maxInternalEdges = 6, maxExternalEdges = 3, minimumTotalEdges = 0, seed = Option(1))
    val refSetting3 = ParamSetting("Dev2", k = 3, nbVertices = 1000, dev = Array(7D, 10D), theta = 0D, nbRep = defNbRep,
        maxInternalEdges = 6, maxExternalEdges = 3, minimumTotalEdges = 0, seed = Option(1))
    val refSetting4 = ParamSetting("Wth1", k = 3, nbVertices = 1000, dev = Array(7D, 1D), theta = 0D, nbRep = defNbRep,
        maxInternalEdges = 10, maxExternalEdges = 3, minimumTotalEdges = 0, seed = Option(1))
    val refSetting5 = ParamSetting("Wth2", k = 3, nbVertices = 1000, dev = Array(7D, 1D), theta = 0D, nbRep = defNbRep,
        maxInternalEdges = 14, maxExternalEdges = 3, minimumTotalEdges = 0, seed = Option(1))
    val refSetting18 = ParamSetting("Wth3", k = 3, nbVertices = 1000, dev = Array(7D, 1D), theta = 0D, nbRep = defNbRep,
        maxInternalEdges = 20, maxExternalEdges = 3, minimumTotalEdges = 0, seed = Option(1))
    
        
    val refSetting6 = ParamSetting("MTE1", k = 3, nbVertices = 1000, dev = Array(7D, 1D), theta = 0D, nbRep = defNbRep,
        maxInternalEdges = 6, maxExternalEdges = 3, minimumTotalEdges = 5000, seed = Option(1))
    val refSetting7 = ParamSetting("MTE2", k = 3, nbVertices = 1000, dev = Array(7D, 1D), theta = 0D, nbRep = defNbRep,
        maxInternalEdges = 6, maxExternalEdges = 3, minimumTotalEdges = 10000, seed = Option(1))
    val refSetting8 = ParamSetting("Btw1", k = 3, nbVertices = 1000, dev = Array(7D, 1D), theta = 0D, nbRep = defNbRep,
        maxInternalEdges = 14, maxExternalEdges = 0, minimumTotalEdges = 0, seed = Option(1))
    val refSetting9 = ParamSetting("Btw2", k = 3, nbVertices = 1000, dev = Array(7D, 1D), theta = 0D, nbRep = defNbRep,
        maxInternalEdges = 14, maxExternalEdges = 3, minimumTotalEdges = 0, seed = Option(1))
    val refSetting10 = ParamSetting("Btw3", k = 3, nbVertices = 1000, dev = Array(7D, 1D), theta = 0D, nbRep = defNbRep,
        maxInternalEdges = 14, maxExternalEdges = 12, minimumTotalEdges = 0, seed = Option(1))
    val refSetting16 = ParamSetting("Both", k = 3, nbVertices = 1000, dev = Array(7D, 1D), theta = 0.5D, nbRep = defNbRep,
        maxInternalEdges = 14, maxExternalEdges = 12, minimumTotalEdges = 0, seed = Option(1))

    val refSetting11 = ParamSetting("Vert1", k = 3, nbVertices = 1500, dev = Array(7D, 1D), theta = 0D, nbRep = defNbRep,
        maxInternalEdges = 6, maxExternalEdges = 3, minimumTotalEdges = 0, seed = Option(1))
    val refSetting12 = ParamSetting("Vert2", k = 3, nbVertices = 2000, dev = Array(7D, 1D), theta = 0D, nbRep = defNbRep,
        maxInternalEdges = 6, maxExternalEdges = 3, minimumTotalEdges = 0, seed = Option(1))
    val refSetting13 = ParamSetting("$\\theta_1$", k = 3, nbVertices = 1000, dev = Array(7D, 1D), theta = 0.1D, nbRep = defNbRep,
        maxInternalEdges = 6, maxExternalEdges = 3, minimumTotalEdges = 0, seed = Option(1))
    val refSetting14 = ParamSetting("$\\theta_2$", k = 3, nbVertices = 1000, dev = Array(7D, 1D), theta = 0.3D, nbRep = defNbRep,
        maxInternalEdges = 6, maxExternalEdges = 3, minimumTotalEdges = 0, seed = Option(1))
    val refSetting15 = ParamSetting("$\\theta_3$", k = 3, nbVertices = 1000, dev = Array(7D, 1D), theta = 0.5D, nbRep = defNbRep,
        maxInternalEdges = 6, maxExternalEdges = 3, minimumTotalEdges = 0, seed = Option(1))
    val refSetting17 = ParamSetting("$\\theta_4$", k = 3, nbVertices = 1000, dev = Array(7D, 1D), theta = 1D, nbRep = defNbRep,
        maxInternalEdges = 6, maxExternalEdges = 3, minimumTotalEdges = 0, seed = Option(1))

    val refSetting19 = ParamSetting("Both2", k = 3, nbVertices = 2000, dev = Array(7D, 1D), theta = 0D, nbRep = defNbRep,
        maxInternalEdges = 6, maxExternalEdges = 3, minimumTotalEdges = 20000, seed = Option(1))

        
    val paramListRows = Array(Array(refSetting, refSetting13, refSetting14, refSetting15, refSetting17, refSetting8, refSetting9, refSetting10, refSetting16), 
            Array(refSetting, refSetting4, refSetting5, refSetting18, refSetting6, refSetting7, refSetting11, refSetting12, refSetting19))
    
    val paramNames = List("k", "Nb. Vert.", "Mean 1", "Mean 2", "Std. dev. 1", "Std. dev. 2", "Theta", "Nb. Rep.", "Wth. edges", "Btw. edges", "Min. edges")
    
    def getParamValue(setting: ParamSetting, paramaterName: String) = {
        paramaterName match {
            case "k" => setting.k.toString
            case "Nb. Vert." => setting.nbVertices.toString
            case "Nb. Rep." => setting.nbRep.toString
            case "Std. dev. 1" => setting.dev(0).toString
            case "Std. dev. 2" => setting.dev(1).toString
            case "Theta" => setting.theta.toString
            case "Wth. edges" => setting.maxInternalEdges.toString
            case "Btw. edges" => setting.maxExternalEdges.toString
            case "Min. edges" => setting.minimumTotalEdges.toString
        }
    }

    def printParameters() = {
//        val paramListRows = paramList.splitAt(8)
        latex.print("\\begin{table}")
        latex.print("\\begin{center}")
        latex.print("\\begin{tabular}{l" + ("c" * paramListRows(0).size) + "}")
        
        for(pList <- paramListRows) {
            latex.print("\\toprule")
	        latex.print((for (p <- pList) yield { p.name }).mkString(" & ", " & ", " \\\\"))
	        latex.print("\\midrule")
	        latex.print("\\midrule")
	        for (paramName <- paramNames) {
	            val refValue = getParamValue(refSetting, paramName)
	            latex.print((for (p <- pList) yield {
	                val curValue = getParamValue(p, paramName)
	                if (refValue != curValue) { "\\textbf{" + curValue + "}" } else { curValue }
	            }).mkString(paramName + " & ", " & ", "\\\\"))
	        }
	        latex.print("\\bottomrule")
        }
        
        latex.print("\\end{tabular}")
        latex.print("\\caption{Measures}")
        latex.print("\\end{center}")
        latex.print("\\end{table}")
    }
    
    def main(args: Array[String]): Unit = {
        
        
        printParameters()
        
        // Perform the experiments
//        val results = for (params <- paramListRows.flatten) yield { performXp(params) }
//        Serializer.serialize(results, "resultsXp1")
//        val results : Array[HashMap[String, Double]] = Serializer.unserialize("resultsXp1")
        
        // Measures
        latex.print("\\begin{table}")
        latex.print("\\begin{center}")
        latex.print("\\begin{tabular}{l" + ("c" * paramListRows(0).size) + "}")
        
        for((pList,idx) <- paramListRows.zipWithIndex) {
            val results = for (params <- pList) yield { performXp(params) }
            
            Serializer.serialize(results, "resultsXp" + idx)
//            val results : Array[HashMap[String, Double]] = Serializer.unserialize("resultsXp" + idx)
            
            val referenceResult = results(0)
            latex.print("\\toprule")
	        latex.print((for (p <- pList) yield { p.name }).mkString(" & ", " & ", " \\\\"))
	        latex.print("\\midrule")
	        for ((measureCategory, x) <- Measures.measuresByType; if (measureCategory != "Other Measures")) {
	            latex.print("\\midrule")
	            latex.print("\\multicolumn{8}{l}{" + measureCategory + "} \\\\")
	            latex.print("\\midrule")
	            for (measure <- x; if (measures.contains(measure))) {
	                latex.print((for (r <- results) yield {
                        val observedResult = Xp.formatDouble(r(measure.toString))
                        if(Xp.formatDouble(referenceResult(measure.toString)) == observedResult) observedResult else {
                            "\\textbf{" + observedResult + "}"
                        }
                        
                    }).mkString(measure.toString + " & ", " & ", "\\\\"))
	            }
	        }
	        latex.print("\\bottomrule")
        }
        latex.print("\\end{tabular}")
        latex.print("\\caption{Measures computed for the graph generated according the parameters presented Table~\\ref{tab:settings}}")
        latex.print("\\end{center}")
        latex.print("\\end{table}")
        latex.printEnding()
        latex.compile
    }

    def performXp(params: ParamSetting) = {
        println(params)
        val outPath = basePath + params.name + "/"
        new File(outPath).mkdirs()
        val gen = new Generator1(k = params.k, nbVertices = params.nbVertices, dev = params.dev, theta = params.theta, nbRep = params.nbRep,
            maxInternalEdges = params.maxInternalEdges, maxExternalEdges = params.maxExternalEdges, minimumTotalEdges = params.minimumTotalEdges, seed = params.seed, probaCloseTriangle = 0)
        val m = new GraphMeasures(gen.graph, params.dev)
        val comInformation = new CommunitiesInformation(gen).communitiesInformation

        val out = new Output(gen)
        out.outputCSV(s"out_graph/${params.name}")
        
//        out.outputDegreeDistribution(s"${outPath}degDist.pdf", m.degreeDistribution)
//        for (i <- 0 until params.k) { out.outputDegreeDistribution(s"${outPath}degDistCom${i}.pdf", comInformation(i).degreeDistribution) }

        //        out.outputPlotXY(s"${outPath}plotXY.pdf")
        //        out.toJungGraph(s"${outPath}graphFRLayout.pdf")

        // Communities statistics
//        latex.println("\\begin{table}")
//        latex.println("\\begin{center}")
//        latex.println("\\begin{tabular}{lccc}")
//        latex.println("\\toprule")
//        latex.println("& Nb Vertices & Within Edges & Between Edges \\\\")
//        latex.println("\\midrule \\midrule")
//        for (i <- 0 until params.k) {
//            val info = comInformation(i)
//            latex.println(s"Community ${i + 1} & ${info.vertices.size} & ${info.nbInternalEdges} & ${info.nbExternalEdges} \\\\")
//        }
//        latex.println("\\bottomrule")
//        latex.println("\\end{tabular}")
//        latex.println("\\caption{Statistics per community}")
//        latex.println("\\end{center}")
//        latex.println("\\end{table}")

        
        
        
        
        
        // Degree distribution
        //        latex.println("\\begin{figure}")
        //        latex.println(s"\\newcommand{\\w}{${0.9D/(params.k + 1)}\\linewidth}")
        //        latex.println("\\subfigure[Generated graph]{\\includegraphics[width=\\w]{" + params.name + "/degDist.pdf}}")
        //        for(i <- 0 until params.k) {
        //        	latex.println("\\subfigure[Community " + (i + 1) + "]{\\includegraphics[width=\\w]{" + params.name + "/degDistCom" + i + ".pdf}}")
        //        }
        //        latex.println("\\subfigure[Graph layout using Fruchterman-Reingold algorithm]{\\includegraphics[width=.45\\linewidth]{" + params.name + "/graphFRLayout.pdf}}")
        //        latex.println("\\hspace{10pt}")
        //        latex.println("\\subfigure[XY plot of the vertices numeric attributes]{\\includegraphics[width=.45\\linewidth]{" + params.name + "/plotXY.pdf}}")
        //        latex.println("\\caption{Degree distributions (first row), graph edges (bottom left) and attributes (bottom right)}")
        //        latex.println("\\end{figure}")

        //        // Graph
        //        latex.println("\\begin{figure}")
        //        latex.println("\\subfigure[Graph layout using Fruchterman-Reingold algorithm]{\\includegraphics[width=.45\\linewidth]{graphFRLayout.pdf}}")
        //        latex.println("\\hspace{10pt}")
        //        latex.println("\\subfigure[XY plot of the vertices numeric attributes]{\\includegraphics[width=.45\\linewidth]{plotXY.pdf}}")
        //        latex.println("\\end{figure}")
        //        

        val measuresResult = HashMap[String, Double]()
        for ((measureCategory, x) <- Measures.measuresByType; if (measureCategory != "Other Measures")) {
            for (measure <- x; if (measures.contains(measure))) {
                measuresResult(measure.toString) = Measures.getValue(measure, m, gen)
            }
            measuresResult("Runtime (sec)") = gen.runTimeSec
        }
        measuresResult
    }
}

case class ParamSetting(name: String, var k: Int, var nbVertices: Int, dev: Array[Double], theta: Double, var nbRep: Int,
    var maxInternalEdges: Int, var maxExternalEdges: Int, var minimumTotalEdges: Int, seed: Option[Long] = Option(1),
    val italicAttrMeasure : Boolean = false, val italicStructMeasure : Boolean = false)
