/**********************************************************************************************************************************************************
Copyright 2014, Christine Largeron, Universit Jean Monnet.
Licensed under the GNU General Public License V3
 
This file is part of DANC_Generator.
http://perso.univ-st-etienne.fr/largeron/DANC_Generator/.

DANC_Generator is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DANC_Generator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DANC_Generator.  If not, see <http://www.gnu.org/licenses/>.

If you publish material based on DANC_Generator, then, please note the assistance you received by using this project.
Use the following reference for referring to this project:
Generating Attributed Networks with Communities by Christine Largeron, Pierre-Nicolas Mougel, Reihaneh Rabbany and Osmar R. Zaane published in PLOS ONE.
**********************************************************************************************************************************************************/

package xps

import java.io.File
import java.io.PrintWriter
import algo.Generator1
import utils.FileUtils
import utils.Stat

object XpRuntime {
    case class RuntimeRange(label : String, range : Seq[Int])
    
    // val vertRange = RuntimeRange("nbVertices", List(500, 1000, 5000, 10000, 50000))
    val vertRange = RuntimeRange("nbVertices", 10000 to 100000 by 10000)
    val vert2Range = RuntimeRange("nbVertices_MTE", 10000 to 100000 by 10000)
    val wthEdgesRange = RuntimeRange("wthEdges", 1 to 50 by 5)
    val btwEdgesRange = RuntimeRange("btwEdges", 0 to 50 by 5)
    val nbRepRange = RuntimeRange("nbRep", 100 to 1000 by 100)
    val kRange = RuntimeRange("k", 2 :: (10 to 100 by 10).toList)
    val mteRange = RuntimeRange("MTE", (0 to 100000 by 20000).toList)

    val nbRun = 3

    def main(args: Array[String]) = {
        new File("xp/runtime/").mkdirs()
          val ranges = List(wthEdgesRange, btwEdgesRange, nbRepRange, kRange, mteRange, vertRange, vert2Range)
//        val ranges = List(vertRange)
//        val ranges = List(wthEdgesRange)
        for(range <- ranges) {
            FileUtils.using("xp/runtime/" + range.label) { w =>
	            for (v <- range.range) {
	                var params = ParamSetting("Ref", k = 3, nbVertices = 10000, dev = Array(7D, 1D), theta = 0D, nbRep = 100, 
	                        maxInternalEdges = 6, maxExternalEdges = 3, minimumTotalEdges = 0, seed = Option(1))
	                        
	                range match {
	                    case `vertRange` => {
	                        params.nbVertices = v
	                    }
	                    case `wthEdgesRange` => params.maxInternalEdges = v
	                    case `btwEdgesRange` => { params.maxInternalEdges = 50; params.maxExternalEdges = v }
	                    case `nbRepRange` => params.nbRep = v
	                    case `kRange` => params.k = v
	                    case `mteRange` => params.minimumTotalEdges = v
	                    case `vert2Range` => {
	                        params.nbVertices = v
	                        params.minimumTotalEdges = v * 10 
	                    }
	                }
	                
	                for (seed <- 0 to 4) {
	                new Generator1(k = params.k, nbVertices = params.nbVertices, dev = params.dev, theta = params.theta, probaCloseTriangle = 0, nbRep = params.nbRep,
			                maxInternalEdges = params.maxInternalEdges, maxExternalEdges = params.maxExternalEdges, minimumTotalEdges = params.minimumTotalEdges, seed = Option(0)).runTimeSec.toDouble
	                }
	                
			        val runTimes : Seq[Double] = (for (seed <- 0 to nbRun) yield {
			            new Generator1(k = params.k, nbVertices = params.nbVertices, dev = params.dev, theta = params.theta, probaCloseTriangle = 0, nbRep = params.nbRep,
			                maxInternalEdges = params.maxInternalEdges, maxExternalEdges = params.maxExternalEdges, minimumTotalEdges = params.minimumTotalEdges, seed = Option(seed)).runTimeSec.toDouble
			        })
			        val (mean, dev) = Stat.meanStdDev(runTimes)
			        w.println(s"${v}\t${mean}\t${dev}")
			        println(s"${v}\t${mean}\t${dev}")
	            }
	        }
        }
    }

}

